#! /usr/bin/env python
#
# Copyright 2007  Jochen Voss <voss@seehuhn.de>
#
# This file is part of the CSimGen package.
#
# CSimGen is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301 USA

"""Generate C code.

This module provides the top level C code generation facilities.
Typical usage is to first instantiate a Source object, to add
functions to it using the Source.function() and Source.main() methods.
The result can be saved using Source.save().

Most code generation facilities are implemented as methods of the
function object classes Main and Block.

Exported Classes
----------------

:class:`~csimgen.code.Source`
  a complete C source files
:class:`~csimgen.code.Main`
  the C 'main' function
:class:`~csimgen.code.Block`
  a (normally bracketed) block of C source code
:class:`~csimgen.code.Code`
  abstract base class

WARNING: this is an experimental version of CSimGen.  The API is not
yet finalised and may change in the future.
"""

__author__ = 'Jochen Voss'
__version__ = '0.1'

import os.path
import time

from paths import DATADIR
from value import value, Value, ValueVar
from vector import Vector, CVector, ConstVector, ExplicitVector, FnVector
from matrix import Matrix, CMatrix, ExplicitMatrix, ConstMatrix, FnMatrix
from mgt import Mgt
from mgb import Mgb, CMgb, ConstMgb
from helpers import get_auxfns, get_helpers
from format import split_list, break_list, string_to_C
from error import checkOptArgs

try:
    from os import getenv
    _default_author_name=getenv("CSIMGEN_AUTHOR_NAME")
    if not _default_author_name:
        _default_author_name=getenv("GIT_AUTHOR_NAME")
    if not _default_author_name:
        from os import getuid
        import pwd
        _default_author_name=pwd.getpwuid(getuid()).pw_gecos.split(",")[0]
except:
    _default_author_name=None

try:
    from os import getenv
    _default_author_email=getenv("CSIMGEN_AUTHOR_EMAIL")
    if not _default_author_email:
        _default_author_email=getenv("EMAIL")
    if not _default_author_email:
        _default_author_email=getenv("GIT_AUTHOR_EMAIL")
    if _default_author_email and _default_author_email[-1]!=">":
        _default_author_email="<%s>"%_default_author_email
except:
    _default_author_email=None

csi_prefix = ""

class CodeError(Exception):

    """Base class for all code generation errors."""

    def __init__(self, message):
        self.message = message

class Code(object):

    """Base class for all code generators."""

    _serial = 0
    _idcodes = {}

    def __init__(self, seplines=0):
        self.id = Code._serial
        Code._serial += 1
        self.parent = None
        self.seplines = seplines
        self.head = []
        self.body = []
        self.tail = []
        self.depends = set()
        self.helpers = get_helpers()

    def add(self, obj):
        self.body.append(obj)

    def add_head(self, obj):
        self.head.append(obj)

    def add_tail(self, obj):
        self.tail[0:0]=[obj]

    def _lines(self):
        done = set()
        todo = []
        for items in [self.head, self.body, self.tail]:
            for l in items:
                try:
                    id = l.id
                except:
                    id = None
                dep = set([x[1] for x in self.depends if x[0]==id])
                todo.append((dep,id,l))

                changed = True
                while changed and todo:
                    changed = False
                    newtodo = []
                    for dep,id,t in todo:
                        dep -= done
                        if len(dep) == 0:
                            changed = True
                            yield t
                            done.add(id)
                        else:
                            newtodo.append((dep,id,t))
                    todo = newtodo
        for _,id,l in todo:
            yield l

    def lines(self):
        """A generator which produces all lines of the object's body.

        The method yields a list of strings.  The resulting strings
        are already correctly indented, but have no end-of-line
        character appended.
        """
        first=True
        for l in self._lines():
            if not first:
                for i in range(0, self.seplines): yield ""
                first=True
            if isinstance(l,Code):
                for ll in l.lines():
                    yield ll
                    first=False
            else:
                yield str(l)
                first=False

    def helper(self, name, *args, **kwargs):
        bl = self
        while name not in bl.helpers:
            if not bl.parent:
                raise CodeError("invalid helper %s"%name)
            bl = bl.parent
        bl.helpers[name](self, *args, **kwargs)

class Block(Code):

    """A (bracketed) block of C code."""

    def __init__ (self, parent, indent=0, first="{", last="}",
                  position="body"):
        Code.__init__(self)

        self.parent = parent
        self.indent = indent
        self.first = first
        self.last = last

        if isinstance(parent,Source):
            self.source = parent
        else:
            self.source = parent.source
        { "body": parent.body,
          "head": parent.head,
          "tail": parent.tail }[position].append(self)

        self.definitions = []
        self.initializers = []
        self.destructors = []
        self.ret = None
        self.symbols = {}

    def lines(self, pfx=None):
        indent = self.indent
        if pfx is None: pfx = "  "*indent
        if self.first is not None: yield pfx+self.first
        if self.definitions:
            def gather():
                last,spec,linit = self.definitions[0]
                specs = [spec]
                for target,spec,init in self.definitions[1:]:
                    if target == last and init is None and linit is None:
                        specs.append (spec)
                    else:
                        yield (last,specs,linit)
                        last = target
                        specs = [spec]
                    linit = init
                yield (last,specs,linit)
            for t,specs,init in gather():
                if t is None: continue
                if init is not None:
                    assert len(specs) == 1
                    if isinstance (init,basestring) and "//" in init:
                        ii = init.split("//")[:-1]
                        yield pfx+"  "+t+"  "+specs[0]+" = {"
                        for i in ii: yield pfx+"    "+i
                        yield pfx+"  };"
                    elif isinstance (init,list):
                        yield pfx+"  "+t+"  "+specs[0]+" = {"
                        for l in break_list (init, pfx+"    "): yield l
                        yield pfx+"  };"
                    else:
                        yield pfx+"  "+t+"  "+specs[0]+" = "+str(init)+";"
                else:
                    for l in split_list (specs, pfx+"  "+t+"  ", ";"):
                        yield l
            yield pfx+"  "
        if self.initializers:
            for l in self.initializers:
                yield pfx+"  "+l
            yield pfx+"  "
        for l in self.body:
            if isinstance (l, Block):
                for ll in l.lines(pfx+"  "): yield ll
            else:
                yield pfx+"  "+l
        if self.destructors:
            yield pfx+"  "
            for l in self.destructors:
                yield pfx+"  "+l
        if self.ret:
            yield pfx+"  "
            yield pfx+"  "+self.ret
        if self.last is not None: yield pfx+self.last

    def __getitem__(self, key):
        return self.symbols[key]

    def _outermost_parent_block(self):
        bl = self
        while bl.parent and isinstance(bl.parent,Block):
            bl = bl.parent
        return  bl

    def exit(self, rc=0):
        self.add_include("<stdlib.h>")
        self.funcall("exit", rc)

    def cassert(self, cond):
        self.add_include("<assert.h>")
        self.funcall("assert", cond)

    def block(self):
        return  Block(self, self.indent+1)

    def cond(self, cond, *clauses):
        cmd = "if (%s)"%cond
        cond = MaybeBracedBlock(self, cmd, self.indent+1)
        cond.body += clauses
        return  cond

    def ifelse(self, *conditions):
        res = []
        c = Block(self, self.indent+1,
                  first="if (%s) {"%conditions[0], last=None)
        res.append(c)
        for cond in conditions[1:]:
            c = Block(self, self.indent+1,
                      first="} else if (%s) {"%cond, last=None)
            res.append(c)
        c = Block(self, self.indent+1, first="} else {", last="}")
        res.append(c)

        return res

    def while_loop(self, cond, *clauses):
        cmd = "while (%s)"%cond
        loop = MaybeBracedBlock(self, cmd, self.indent+1)
        loop.body += clauses
        return  loop

    def loop(self, i, a, b, step=1, compact=True):
        a = value(a).simplify()
        b = value(b).simplify()
        i = value(i)
        if step == 1:
            tmpl = "for (%s=%s; %s<%s; ++%s)"
            cmd = tmpl%(i, a, i, b, i)
        else:
            step = value(step)
            tmpl = "for (%s=%s; %s<%s; %s+=%s)"
            cmd = tmpl%(i, a, i, b, i, step)
        if compact:
            loop = MaybeBracedBlock(self, cmd, self.indent+1)
        else:
            loop = Block(self, self.indent+1, cmd+" {", "}")
        return  loop

    def statement(self, *lines):
        """Add individual lines to the function body."""
        self.body += lines

    def statements(self, lines):
        """Add a multi-line string to the body, fixing the indentation.

        This adds a multi-line string to the function body.  Leading
        white-space for the second and following line is striped to
        align the lines with the surrounding code.  Relative
        indentation of the lines is preserved.  The first line is
        aligned separately to make use with python multi-line strings
        more convenient.
        """
        lines = [l.rstrip().expandtabs() for l in lines.splitlines()]
        if len(lines)==0:
            return
        l0 = lines[0]
        n0 = len(l0)-len(l0.lstrip())
        if len(lines) == 1:
            lines[0] = l0[n0:]
        elif n0 == 0:
            l1 = lines[1]
            n1 = len(l1)-len(l1.lstrip())
            lines[0] = " "*n1+l0
        n = min([len(l)-len(l.lstrip()) for l in lines if l!=""])
        for l in lines:
            self.body.append(l[n:])

    def funcall(self, fn, *args, **kw):
        start = fn+"("
        if "res" in kw:
            start = str(kw["res"]) + " = " + start
        position = "body"
        if "position" in kw:
            position = kw["position"]
        ll = getattr(self, position)
        for l in break_list(map(str,args), start, ");", ",", " ",
                            79-2*(self.indent+1)):
            ll.append(l)

    def add_include(self, header):
        """Add an #include statement to the top of the C source file."""
        self.source.add_include (header)

    def _auxfn(self, func):
        id = self._outermost_parent_block().id
        return self.source._auxfn(func, id)

    def lapack(self, func):
        id = self._outermost_parent_block().id
        return  self.source.lapack (func, id)


    def _gen_name(self, typecode):
        """Generate a new, unused variable name.

        The following type codes are used:
            d - integers except loop indices
            f - doubles
            i - loop indices
            m - matrices
            s - strings
            v - vectors
        """
        tmpl = csi_prefix+typecode
        i = Code._idcodes.get(typecode,0)
        Code._idcodes[typecode] = i+1
        name = tmpl+str(i)
        return  name

    def _loop_index(self,order):
        bl = self._outermost_parent_block()
        if order == 0:
            name = "i"
        elif order == 1:
            name = "j"
        else:
            name = "i%d"%(order-2)
        name = csi_prefix+name
        if name in bl.symbols: return bl.symbols[name]
        return bl.define_int(name=name)

    def add_definition(self, name, type, init=None, len=None):
        name=str(name)
        if len is None:
            self.definitions.append ((type, name, init))
        elif len == 0:
            self.definitions.append ((type, "*"+name, init))
        elif len.is_const_C():
            self.definitions.append ((type, name+"[%s]"%len, init))
        else:
            self.definitions.append ((type, "*"+name, init))
            self.helper("malloc", name, type, len)
            self.helper("free", name, position="destructors")


    def init_noise(self, seed=None):
        """Initialise the random number generator.

        This can be called at any point of the programm.
        """
        if seed is None:
            self.add_include ('<time.h>')
            seed = "time (NULL)"
        self.add_include ('"jvrandom.h"')
        self.initializers.append ("init_noise (%s);"%seed)


    def define_string(self, len, init=None, *args, **kwargs):
        if "name" in kwargs:
            name = kwargs["name"]
            assert name not in self.symbols
        else:
            name = self._gen_name("s")
        self.add_definition (name, "char", None, value(len))
        if init is not None:
            self.add_include ('<stdio.h>')
            self.funcall ("snprintf", name, len, init, *args)
        x = value(name)
        self.symbols[name] = x
        return  x

    def _define_scalar(self, name, type, const, init, early, inline=False):
        x = value(name, known_c=const)
        if init is not None:
            init = value(init)
            if not init.is_const_C() and not inline:
                # TODO: is_const_C is not what we really need
                self.store_scalar(x, init, early)
                init = None
        assert  not (const and init is None)
        if const: type = "const "+type
        self.add_definition(name, type, init)

        assert name not in self.symbols
        self.symbols[name] = x
        return  x

    def define_int(self, init=None, name=None, early=False, inline=False):
        if name is not None:
            assert name not in self.symbols
        else:
            name = self._gen_name("d")
        return  self._define_scalar(name, "int", False, init, early, inline)

    def define_const_int(self, name, init):
        return  self._define_scalar (name, "int", True, init, False)

    def define_long(self, name, init=None, early=False):
        return  self._define_scalar (name, "long", False, init, early)

    def define_ulong(self, name, init=None, early=False):
        return  self._define_scalar (name, "unsigned long", False, init, early)

    def define_double(self, name=None, init=None, const=False):
        if name is not None:
            assert name not in self.symbols
        else:
            name = self._gen_name("f")
        if const and init:
            init = value(str(init), known_c=True)
        return  self._define_scalar(name, "double", False, init, False)

    def define_const_double(self, name, init):
        return  self._define_scalar (name, "double", True, init, False)

    def define_vector(self, n, init=None, name=None):
        if name is not None:
            assert name not in self.symbols
        else:
            name = self._gen_name("v")
        x = CVector (n, name)
        initstr = None
        if isinstance (init,list):
            if len(init) != n:
                tmpl = "wrong number of initialisers (need %d, found %d)"
                raise ValueError(tmpl%(n, len(init)))
            initstr = map(str,init)

        specn,spect,specl = x.spec ()
        self.add_definition (specn, spect, initstr, specl)

        if isinstance (init, (int,float,Value)):
            self.store_vector (x, ConstVector (n, init))
        elif initstr is None and init is not None:
            self.store_vector (x, init)

        self.symbols[name] = x
        return  x

    def define_matrix(self, m, n, init=None, transposed=False, name=None):
        """Define a matrix in the generated C code.

        This adds the necessary definition to store a full matrix in
        the generated program.  `name` is the variable name, `m` is
        the number of rows, `n` the number of columns.  If `init` is
        set, it is used to initialise the matrix; it can either be a
        number (which is then copied into all matrix entries), a
        matrix of the same dimensions, or a list of length `m*n`
        (which gives the elements in row-major order).

        If `transposed` is true, the generated matrix is internally
        stored in column major order.  Changing this flag does not
        functionally change the resulting program, but may affect
        performance.
        """
        if name is not None:
            assert name not in self.symbols
        else:
            name = self._gen_name("m")
        A = CMatrix (m, n, name, transposed)
        if isinstance (init, list):
            init = ExplicitMatrix (m, n, init)
        elif isinstance (init,(int,float,Value)):
            init = ConstMatrix (m, n, init)

        initstr = None
        if isinstance (init,ExplicitMatrix):
            if init.m != m or init.n != n:
                raise  ValueError("Matrix dimensions don't match")
            if transposed: init=init.trans()
            rows = []
            for i in range(0,init.m):
                rows.append ([str(init[i,j]) for j in range(0,init.n)])
            initstr = ",//".join([", ".join(x) for x in rows])+"//"

        specn,spect,specl = A.spec ()
        self.add_definition (specn, spect, initstr, specl)

        if initstr is None:
            if isinstance (init,Matrix):
                self.store_matrix (A, init)
            elif init is not None:
                raise TypeError("cannot initialize Matrix from %s"%type(init))

        self.symbols[name] = A
        return  A

    def define_matrix_from_file(self, fname, name=None):
        """Add code to read a matrix from file.

        The return value is a Matrix, representing the read matrix in
        memory.  The matrix dimensions can be found by inspecting the
        'm' and 'n' fields of the returned object.
        """
        if name is not None:
            assert name not in self.symbols
        else:
            name = self._gen_name("m")
        m = self.define_int()
        n = self.define_int()
        A = CMatrix(m, n, name)
        self.add_definition(A, "double", None, 0)
        self.funcall(self._auxfn("new_matrix_from_file"),
                     fname, "&"+str(m), "&"+str(n), res=A)
        self.helper("free", A, position="destructors")
        self.symbols[name] = A
        return  A

    def define_mgt(self, namel, namem, namer, n, init=None):
        n = value(n)
        self.definitions.append ((None,None,None))
        x = Mgt (n,
                 self.define_vector(n-1, name=namel),
                 self.define_vector(n, name=namem),
                 self.define_vector(n-1, name=namer))
        self.definitions.append ((None,None,None))
        if isinstance (init,(int,float,Value)):
            init = ConstMatrix (n, n, init)
        if init is not None:
            self.store_mgt (x, init)
        return  x

    def define_mgb(self, m, n, kl, ku, init=None, name=None, extra=0):
        """Define a general banded matrix in the generated source.

        Arguments:
        m -- number of matrix rows
        n -- number of matrix columns
        kl -- number of subdiagonals (kl >= 0)
        ku -- number of superdiagonals (ku >= 0)
        init -- if not None: initial value of the matrix
        extra -- number of extra rows for use with 'lusolve_mgb_vector1'
        """
        if name is not None:
            assert name not in self.symbols
        else:
            name = self._gen_name("m")
        m = value(m)
        n = value(n)
        kl = value(kl)
        ku = value(ku)
        x = CMgb (m, n, kl, ku, name, extra)
        specn,spect,specl = x.spec ()
        self.add_definition (specn, spect, None, specl)
        if isinstance(init, list):
            init = ExplicitMatrix(m, n, init)
        elif isinstance(init, (int,float,Value)):
            init = ConstMatrix(m, n, init)
        if init is not None:
            self.store_mgb (x, init)
        return  x

    def define_file(self, name, fname=None, fmode="r"):
        """Open a file in the C program.

        Create a C variable `name` of type `FILE *`, add code to open
        the file 'fname' and assign the file handle to 'name'.  The
        return value is the Python representation of 'name'.

        Note that 'fname' is directly used as the first argument of
        the 'fopen' call.  If you want to give the file name
        explicitly, you will need two sets of quote character, one for
        python and one for C.  Example:

            fd = block.define_file("fd",'"matrix.dat"')

        After use, the file must be closed with 'Block.close_file()'.
        """
        x = value(name)
        self.add_include ('<stdio.h>')
        self.add_definition (name, "FILE", None, 0)
        if fname:
            self.funcall ("fopen", fname, '"%s"'%fmode, res=x)
        return  x

    def close_file(self, fd):
        self.add_include ('<stdio.h>')
        self.funcall ("fclose", fd)

    def _update_scalar(self, RES, X, op="="):
        self.body.append ("%s %s %s;"%(RES,op,X))

    def _update_vector(self, RES, X, op="="):
        n = value(RES.n)
        if X.n != n:
            tmpl = "vectors have different lengths (%s vs. %s)"
            raise ValueError(tmpl%(X.n,n))

        keys = RES.special() | X.special()
        if value(1) in keys:
            keys.add(value(0))
        if n-2 in keys:
            keys.add(n-1)

        done = set()
        a = value(0)
        while a in keys:
            tmpl = str(RES[a])+" "+op+" "+str(X[a])+";"
            self.body.append (tmpl)
            done.add (a)
            a += 1

        tail = []
        b = n-1
        while b in keys:
            if b < a: break
            tmpl = str(RES[b])+" "+op+" "+str(X[b])+";"
            tail.append (tmpl)
            done.add (b)
            b -= 1

        if not b < a:
            todo = keys - done
            if len(todo)>0:
                raise NotImplementedError()
            i = self._loop_index(0)
            loop = self.loop(i, a, b+1)
            loop._update_scalar(RES[i], X[i], op)

        for l in reversed(tail):
            self.body.append (l)

    def _update_matrix(self, RES, X, op="="):
        m = RES.m.simplify()
        n = RES.n.simplify()
        assert X.m == m and X.n == n
        if m == 0 or n == 0: return
        if isinstance(X, ExplicitMatrix):
            for i in range(0, m):
                for j in range(0, n):
                    self._update_scalar(RES[i,j], X.get(i,j,False), op)
        elif getattr(RES, "transposed", False):
            if n == 1:
                j = 0
                lj = self
            else:
                j = self._loop_index(1)
                lj = self.loop(j, 0, n, compact=False)
            if m == 1:
                i = 0
                li = lj
            else:
                i = self._loop_index(0)
                li = lj.loop(i, 0, m, compact=False)
            li._update_scalar(RES[i,j], X.get(i,j,False), op)
        else:
            if m == 1:
                i = 0
                li = self
            else:
                i = self._loop_index(0)
                li = self.loop(i, 0, m, compact=False)
            if n == 1:
                j = 0
                lj = li
            else:
                j = self._loop_index(1)
                lj = li.loop(j, 0, n, compact=False)
            lj._update_scalar(RES[i,j], X.get(i,j,False), op)

    def _update_mgt(self, RES, X, op="="):
        n = RES.n.simplify()
        assert X.n == n
        i = self._loop_index(0)
        loop = self.loop (i, 0, n-1)
        loop._update_scalar (RES[i+1,i], X[i+1,i], op)
        loop = self.loop (i, 0, n)
        loop._update_scalar (RES[i,i], X[i,i], op)
        loop = self.loop (i, 0, n-1)
        loop._update_scalar (RES[i,i+1], X[i,i+1], op)

    def _update_mgb(self, RES, X, op="="):
        m = RES.m.simplify()
        n = RES.n.simplify()
        kl = RES.kl
        ku = RES.ku
        assert X.m == m and X.n == n
        i = self._loop_index(0)
        j = self._loop_index(1)
        l1 = self.loop(i, 0, m)
        l2 = l1.loop(j, i-kl, i+ku+1)
        l2.cond("%s<0 || %s>=%s"%(j, j, n),
                "continue;")
        l2._update_scalar(RES[i,j], X[i,j], op)

    def store_scalar(self, RES, X, early=False):
        s = "%s = %s;"%(RES,X)
        if early:
            self.initializers.append (s)
        else:
            self.body.append (s)

    store_double = store_scalar

    def store_vector(self, RES, X):
        """Store a copy of X in RES.

        X must be a Vector and RES must be a CVector.
        """
        res_base,res_inc = RES.base_and_inc()
        try:
            x_base,x_inc = X.base_and_inc()
        except AttributeError:
            x_inc = None
        if res_inc == 1 and x_inc == 1:
            n = RES.n
            assert X.n == n
            self.add_include ('<string.h>')
            self.funcall("memcpy", res_base, x_base, n*value("sizeof(double)"))
        else:
            self._update_vector (RES, X, "=")

    def store_matrix(self, RES, X):
        """Store a copy of X in RES.

        X must be a Matrix and RES must be a CMatrix.
        """
        if (isinstance (X, CMatrix)
            and isinstance (RES, CMatrix)
            and RES.transposed == X.transposed):
            m = RES.m
            n = RES.n
            assert X.m == m and X.n == n
            self.add_include ('<string.h>')
            tmpl = "memcpy (%s, %s, %s*sizeof(double));"
            self.statement (tmpl%(RES.base,X,(m*n).protect(3)))
        else:
            self._update_matrix (RES, X, "=")

    def store_mgt(self, RES, X):
        """Emit code to store a copy of X in RES."""
        if isinstance (X, Mgt):
            n = RES.n
            assert X.n == n
            self.store_vector (RES.datal, X.datal)
            self.store_vector (RES.datam, X.datam)
            self.store_vector (RES.datar, X.datar)
        else:
            self._update_mgt (RES, X, "=")

    def store_mgb(self, RES, X):
        """Emit code to store a copy of X in RES.

        Arguments:
        RES -- A banded matrix in the C program
        X -- A banded matrix
        """
        if isinstance (X, Mgb):
            m = RES.m
            n = RES.n
            kl = RES.kl
            ku = RES.ku
            assert X.m == m and X.n == n and X.kl == kl and X.ku == ku
            if RES.extra != X.extra:
                self.store_matrix (RES.data[RES.extra:,:], X.data[X.extra:,:])
            else:
                self.store_matrix (RES.data, X.data)
        else:
            self._update_mgb (RES, X, "=")

    def iadd_int(self, RES, X=1):
        """Add code to increment 'RES' by 'X'."""
        if X == 0:
            pass
        elif X == 1:
            self.statement ("++%s;"%RES)
        else:
            self._update_scalar (RES, X, "+=")

    def iadd_double(self, RES, X):
        """Add code to increment 'RES' by 'X'."""
        if X == 0:
            pass
        else:
            self._update_scalar (RES, X, "+=")

    def iadd_vector(self, RES, X):
        """Add code to increment 'RES' by 'X'."""
        self._update_vector (RES, X, "+=")

    def iadd_matrix(self, RES, X):
        """Add code to increment 'RES' by 'X'."""
        self._update_matrix (RES, X, "+=")

    def isub_double(self, RES, X):
        """Add code to decrement 'RES' by 'X'."""
        self._update_scalar (RES, X, "-=")

    def isub_vector(self, RES, X):
        """Add code to decrement 'RES' by 'X'."""
        self._update_vector (RES, X, "-=")

    def isub_matrix(self, RES, X):
        """Add code to decrement 'RES' by 'X'."""
        self._update_matrix (RES, X, "-=")

    def imult_double(self, RES, X):
        """Add code to multiply 'RES' by 'X' in place."""
        if X == 0:
            self._update_scalar (RES, 0, "=")
        elif X == 1:
            pass
        else:
            self._update_scalar (RES, X, "*=")

    def imult_vector(self, RES, X):
        self._update_vector (RES, ConstVector(RES.n,X), "*=")

    def imult_matrix(self, RES, X):
        self._update_matrix(RES, ConstMatrix(RES.m,RES.n,X), "*=")

    def imult_mgb(self, RES, X):
        self._update_mgb(RES, ConstMgb(RES.m,RES.n,RES.kl,RES.ku,X), "*=")

    def idiv_double(self, RES, X):
        """Add code to divide 'RES' by 'X'."""
        self._update_scalar(RES, X, "/=")

    def idiv_vector(self, RES, X):
        self._update_vector (RES, ConstVector(RES.n,X), "/=")

    def idiv_matrix(self, RES, X):
        self._update_matrix (RES, ConstMatrix(RES.m,RES.n,X), "/=")

    def dot_vector_vector(self, x, y, res=None, name=None):
        """Compute the Euclidean inner product of 'x' and 'y'.

        If res is set, it must be a variable of type 'double' to store
        the result.  Otherwise the return value is a newly allocated
        double which holds the result.
        """
        if res is None:
            res = self.define_double(name=name)
        if x.n != y.n:
            raise ValueError("X and Y must be of equal length")
        try:
            xbase,xinc = x.base_and_inc()
            ybase,yinc = y.base_and_inc()
            self.funcall(self.lapack("ddot"), x.n, xbase, xinc, ybase, yinc,
                         res=res)
        except AttributeError:
            self.store_scalar(res, 0)
            i = self._loop_index(0)
            l = self.loop(i, 0, x.n)
            l.iadd_double(res, x[i]*y[i])
        return res

    def fmadd_matrix_vector(self, alpha, A, x, beta, y):
        """Emit code for y = alpha A x + beta y.

        A is a CMatrix, x, y are vectors and alpha, beta are numbers.
        """
        m = A.m
        n = A.n
        assert x.n == n and y.n == m
        if m == 0 or n == 0:
            return
        if isinstance(A, CMatrix):
            xbase,xinc = x.base_and_inc()
            ybase,yinc = y.base_and_inc()
            dgemv = self.lapack("dgemv")
            if A.transposed:
                # matrix is stored in Fortran array conventions
                self.funcall(dgemv, "'N'", m, n, alpha, A, m, xbase, xinc,
                             beta, ybase, yinc)
            else:
                # matrix is stored in C array conventions
                self.funcall(dgemv, "'T'", n, m, alpha, A, n, xbase, xinc,
                             beta, ybase, yinc)
        else:
            if m == 1:
                i = 0
                li = self
            else:
                i = self._loop_index(0)
                li = self.loop(i, 0, m)
            li.imult_double(y[i], beta)
            if n == 1:
                j = 0
                lj = li
            else:
                j = self._loop_index(1)
                lj = li.loop(j, 0, n)
            lj.iadd_double(y[i], alpha*A[i,j]*x[j])

    def fmadd_mgb_vector(self, alpha, A, x, beta, y):
        """Emit code for y = alpha A x + beta y.

        A is a CMgb, x, y are vectors and alpha, beta are numbers.
        """
        m = A.m
        n = A.n
        kl = A.kl
        ku = A.ku
        assert x.n == n and y.n == m
        xbase,xinc = x.base_and_inc()
        ybase,yinc = y.base_and_inc()
        self.funcall(self.lapack("dgbmv"), "'N'", m, n, kl, ku,
                     alpha, value(str(A.data))+value(A.extra), A.data.m, xbase,
                     xinc, beta, ybase, yinc)

    def fmadd_matrix_matrix(self, alpha, A, B, beta, C):
        """Emit code for C = alpha A B + beta C.

        A, B, C are matrices and alpha, beta are numbers.
        """
        if A.n != B.m or A.m != C.m or B.n != C.n:
            raise ValueError("incompatible matrix dimensions")

        if A.transposed:
            transP = True
            ldP = A.m
        else:
            transP = False
            ldP = A.n
        if B.transposed:
            transQ = True
            ldQ = B.m
        else:
            transQ = False
            ldQ = B.n

        if C.transposed:
            # C is stored in Fortran array conventions, everything fine.
            P, Q, R = A, B, C
            ldR = C.m
            m, n, k = A.m, B.n, A.n
        else:
            # C is stored in C array conventions: ask dgemm to calculate
            # B^T A^T instead.
            P, Q, R = B, A, C
            transP, transQ = not transQ, not transP
            ldP, ldQ = ldQ, ldP
            ldR = C.n
            m, n, k = B.n, A.m, B.m

        transP = transP and "'N'" or "'T'"
        transQ = transQ and "'N'" or "'T'"

        self.funcall (self.lapack("dgemm"), transP, transQ, m, n, k,
                      alpha, P, ldP, Q, ldQ, beta, R, ldR)


    def norm2_vector(self, x, res=None, name=None):
        if res is None:
            res = self.define_double(name)
        try:
            xbase,xinc = x.base_and_inc ()
            self.funcall(self.lapack("dnrm2"), x.n, xbase, xinc, res=res)
        except AttributeError:
            self.store_scalar(res, 0)
            i = self._loop_index(0)
            l = self.loop(i, 0, x.n)
            l.iadd_double(res, x[i]*x[i])
            self.store_scalar(res, self.sqrt(res))
        return res

    def normsup_vector(self, x, res=None, name=None):
        """compute the infinity-norm of x.

        Store the maximum modulus of the elements of 'x' in 'res'.
        """
        if res is None:
            res = self.define_double(name)
        self.store_scalar(res, x[0])
        i = self._loop_index(0)
        l = self.loop(i, 1, x.n)
        tmp = l.define_double(init=l.fabs(x[i]))
        cond = l.cond("%s > %s"%(tmp,res))
        cond.store_scalar(res, tmp)
        return res

    def getmax_vector(self, x, res=None, name=None):
        if res is None:
            res = self.define_double(name)
        self.store_scalar (res, x[0])
        i = self._loop_index(0)
        l = self.loop (i, 1, x.n)
        cond = l.cond ("%s > %s"%(x[i],res))
        cond.store_scalar (res, x[i])
        return res

    def normfro_matrix(self, A, res=None, name=None):
        """Compute the Frobenius norm of the matrix 'A'.

        This method adds code to compute the Frobenius norm of a
        matrix, i.e. the square root of the sum of the squared
        components of A.  The result is stored in 'res'.
        """
        if res is None:
            res = self.define_double(name)
        if isinstance(A, CMatrix):
            self.funcall(self.lapack("dnrm2"), A.m*A.n, A.base, 1, res=res)
        else:
            self.store_scalar(res, 0)
            m = A.m.simplify()
            n = A.n.simplify()
            i = self._loop_index(0)
            li = self.loop(i, 0, m, compact=False)
            j = self._loop_index(1)
            lj = li.loop(j, 0, n, compact=False)
            tmp = lj.define_double(init=A[i,j])
            lj.iadd_double(res, tmp*tmp)
            self.store_scalar(res, self.sqrt(res))
        return res

    def normsup_matrix(self, A, res=None, name=None):
        """compute the supremum norm of A.

        Store the maximum modulus of the elements of 'A' in 'res'.
        """
        if res is None:
            res = self.define_double(name)
        self.store_scalar(res, self.fabs(A[0,0]))
        m = A.m.simplify()
        n = A.n.simplify()
        i = self._loop_index(0)
        li = self.loop(i, 0, m)
        j = li._loop_index(1)
        lj = li.loop(j, 0, n)
        tmp = lj.define_double(init=lj.fabs(A[i,j]))
        cond = lj.cond("%s > %s"%(tmp,res))
        cond.store_scalar(res, tmp)
        return res

    def iadd_tensor_matrix_vector_vector(self, A, alpha, x, y):
        """Emit code for A += alpha*x*y'.

        A must be a Matrix, alpha a scalar, and x and y are vectors.
        """
        xbase,xinc = x.base_and_inc ()
        ybase,yinc = y.base_and_inc ()
        if A.transposed:
            self.funcall(self.lapack("dger"),
                         A.m, A.n, alpha, xbase, xinc, ybase, yinc, A, A.m)
        else:
            self.funcall(self.lapack("dger"),
                         A.n, A.m, alpha, ybase, yinc, xbase, xinc, A, A.n)

    def invert_matrix(self, A):
        """Emit code to invert the matrix A.

        A is overwritten with the result.
        """
        if A.m != A.n:
            raise CodeError("cannot invert non-square matrix")
        else:
            if not (A.m == A.n):
                self.cassert("%s == %s"%(A.m,A.n))

        n = A.n
        bl = self.block()
        bl.add_definition ("ipiv", "long", None, n)
        bl.funcall (bl.lapack("dgetrf"), n, n, A, n, "ipiv")

        if n <= 16:
            lwork = n
        else:
            tmp = bl.define_double ("tmp")
            bl.funcall (bl.lapack("dgetri"), n, A, n, "ipiv", "&tmp", -1)
            lwork = bl.define_long ("lwork", tmp+0.5)
        work = bl.define_vector(lwork, name="work")
        bl.funcall (bl.lapack("dgetri"), n, A, n, "ipiv", work, lwork)

    def solve_matrix_vector(self, A, X):
        """Emit code to solve A*RES = X for a full matrix A.

        A must be a Matrix and X must be a CVector.
        The vector X it is overwritten with the result,
        the matrix A is not modified.
        """
        n = X.n
        assert A.m == n and A.n == n
        bl = self.block()
        Atmp = bl.define_matrix(n, n, init=A, transposed=True)
        bl.add_definition ("ipiv", "long", None, n)
        xbase,xinc = X.base_and_inc ()
        assert xinc == 1
        bl.funcall (bl.lapack("dgesv"), n, 1, Atmp, n, "ipiv", xbase, n)

    def solve_mgt_vector(self, A, b):
        """Emit code to solve A*RES = b for a general tri-band matrix A.

        A must be a CMgt.  'b' can be either a
        vector or a matrix.  If 'b' is a matrix with n columns, then n
        systems of linear equations are solved simultaneously.

        On exit, 'b' is overwritten with the result RES.  The matrix 'A'
        is not modified.
        """
        n = A.n
        assert b.n == n
        bl = self.block()
        if isinstance(b, CVector):
            base,inc = b.base_and_inc()
            if inc == 1:
                b = CMatrix(b.n, 1, base, transposed=True)
            else:
                raise NotImplementedError()
        elif isinstance(b, CMatrix):
            if not b.transposed:
                raise ValueError("wrong memory layout for b")
        else:
            raise TypeError("invalid argument type '%s' for rhs"%type(b))
        bl.define_mgt("El", "Em", "Er", n, A)
        bl.funcall(bl.lapack("dgtsv"), n, b.n, "El", "Em", "Er", b, b.m)

    def solve_mgb_vector(self, A, b, inplace=False):
        """Emit code to solve :math:`Ax = b` for a general banded matrix :math:`A`.

        A must be a CMgb and 'b' can be either a vector or a matrix.
        If 'b' is a matrix with n columns, then n systems of linear
        equations are solved simultaneously.

        A banded matrix can be inverted by calling this method,
        passing the indentity matrix for 'b'.

        On exit, 'b' is overwritten with the result 'x'.  The matrix
        'A' is not modified.
        """
        n = A.n
        kl = A.kl
        ku = A.ku
        if A.m != n:
            raise ValueError("matrix not square")
        if b.n != n:
            raise ValueError("matrix and vector dimensions don't match")

        bl = self.block()
        if inplace:
            if A.extra != kl:
                raise ValueError("matrix A needs extra=%d"%kl)
        else:
            A = bl.define_mgb(n, n, kl, ku, init=A, extra=kl)
        if isinstance(b, CVector):
            base,inc = b.base_and_inc()
            if inc == 1:
                b = CMatrix(b.n, 1, base, transposed=True)
                tmp = b
            else:
                raise NotImplementedError()
        elif isinstance(b, CMatrix):
            if b.transposed:
                tmp = b
            else:
                if inplace:
                    raise ValueError("B.transposed is false")
                raise NotImplementedError("the current implementation is wrong")
                tmp = bl.define_matrix(b.m, b.n, init=b, transposed=True)
        else:
            raise TypeError("invalid argument type '%s' for rhs"%type(b))
        if not isinstance(A, CMgb):
            raise TypeError("invalid argument type '%s' for matrix"%type(A))

        bl.add_definition ("ipiv", "long", None, n)
        bl.funcall(bl.lapack("dgbsv"), n, kl, ku, tmp.n, A,
                   A.data.m, "ipiv", tmp, tmp.m)

    def lusolve_mgb_vector1(self, A):
        """Emit code to solve A*RES = X using LU-factorisation.

        This method is good if the same left-hand side A is used to
        solve several systems of linear equations.
        lusolve_mgb_vector1 can be called once to perform the
        factorisation and then the (cheaper) lusolve_mgb_vector2 is
        used repeatedly to solve the linear system.

        The matrix A must be a CMgb with kl extra rows.
        On exit it is overwritten with its LU factorisation.
        """
        m = A.m
        n = A.n
        assert A.extra == A.kl
        ipiv_name = A.name+"_ipiv"
        self.add_definition (ipiv_name, "long", None, min(m,n))
        self.funcall (self.lapack("dgbtrf"), m, n, A.kl, A.ku, A,
                      A.data.m, ipiv_name)

    def lusolve_mgb_vector2(self, A, b):
        """Emit code to solve A*x = b using LU-factorisation.

        See the doc string of lusolve_mgb_vector1 for details.

        A must be a CMgb as prepared by lusolve_mgb_vector1 and 'b'
        can be either a vector or a matrix.  If 'b' is a matrix with n
        columns, then n systems of linear equations are solved
        simultaneously.

        On exit, 'b' is overwritten with the result 'x'.
        """
        n = A.n
        if A.m != n:
            raise ValueError("matrix not square")
        if A.extra != A.kl:
            raise ValueError("insufficent extra rows")
        if b.n != n:
            raise ValueError("matrix and vector dimensions don't match")

        bl = self
        if isinstance(b, CVector):
            base,inc = b.base_and_inc()
            if inc == 1:
                b = CMatrix(b.n, 1, base, transposed=True)
                tmp = b
            else:
                raise NotImplementedError()
        elif isinstance(b, CMatrix):
            if b.transposed:
                tmp = b
            else:
                bl = bl.block()
                tmp = bl.define_matrix(b.m, b.n, init=b, transposed=True)
        else:
            raise TypeError("invalid argument type '%s' for rhs"%type(b))
        if not isinstance(A, CMgb):
            raise TypeError("invalid argument type '%s' for matrix"%type(A))

        ipiv_name = A.name+"_ipiv"
        bl.funcall(bl.lapack("dgbtrs"), "'N'", n, A.kl, A.ku, tmp.n,
                   A, A.data.m, ipiv_name, tmp, tmp.m)
        if tmp != b:
            bl.store_matrix(b, tmp)

    def csolve_positive_mgb_vector(self, A, b):
        """Emit code to solve :math:`Ax = b` using Cholesky-factorisation.

        The matrix A must be a symmetric, positive definite CMgb.
        Only the upper half of 'A' is read.  'b' can be either a
        vector or a matrix.  If 'b' is a matrix with n columns, then n
        systems of linear equations are solved simultaneously.

        On exit, 'b' is overwritten with the result 'x'.
        """
        n = A.n
        if A.m != n:
            raise ValueError("matrix not square")

        bl = self
        if isinstance(b, CVector):
            base,inc = b.base_and_inc()
            if inc == 1:
                b = CMatrix(b.n, 1, base, transposed=True)
                tmp = b
            else:
                raise NotImplementedError()
        elif isinstance(b, CMatrix):
            if b.transposed:
                tmp = b
            else:
                bl = bl.block()
                tmp = bl.define_matrix(b.m, b.n, init=b, transposed=True)
        else:
            raise TypeError("invalid argument type '%s' for rhs"%type(b))
        if not isinstance(A, CMgb):
            raise TypeError("invalid argument type '%s' for matrix"%type(A))
        if tmp.m != n:
            raise ValueError("matrix and vector dimensions don't match")

        bl.funcall(bl.lapack("dpbsv"), "'U'", n, A.ku, tmp.n,
                   A.data, A.data.m, tmp, tmp.m)
        if tmp != b:
            bl.store_matrix(b, tmp)

    def csolve_positive_mgb_vector1(self, A):
        """Emit code to solve A*RES = X using Cholesky factorisation.

        This method is good if the same left-hand side A is used to
        solve several systems of linear equations.
        csolve_positive_mgb_vector1 can be called once to perform the
        factorisation and then the (cheaper) function
        csolve_positive_mgb_vector2 is used repeatedly to solve the
        linear system.

        The matrix A must be a symmetric, positive definite CMgb and X
        must be a CVector.  Only the upper half of A is used in this
        function.  The matrix A is overwritten with its Cholesky
        factorisation.

        See also lusolve_mgb_vector1 for a method which works for
        general banded matrices.
        """
        if A.m != A.n:
            raise ValueError("matrix must be square")
        self.funcall (self.lapack("dpbtrf"), "'U'", A.n, A.ku, A, A.data.m)

    def csolve_positive_mgb_vector2(self, A, X):
        """Emit code to solve A*RES = X using Cholseky factorisation.

        See the doc string of csolve_positive_mgb_vector1 for details.

        A must be a CMgb as prepared by csolve_positive_mgb_vector1.
        Only the upper half of A is used in this function.  The vector
        X it is overwritten with the result.
        """
        if A.m != A.n:
            raise ValueError("matrix must be square")
        self.funcall (self.lapack("dpbtrs"), "'U'", A.n, A.ku, 1,
                      A, A.data.m, X, A.n)

    def least_squares_matrix_vector(self, A, b):
        """Emit code to solve the least squares problem :math:`|Ax - b| = min`.

        'A' must be a matrix in the C programm and 'b' can be either a
        vector or a matrix.  If 'b' is a matrix with n columns, then n
        least squares problems are solved simultaneously.  If the C
        representation of 'b' has more rows than 'A' (this might be
        required to store the result), then the input data must be
        stored in the first rows, the excess rows are never read.

        If 'b' is a matrix, for maximal efficiency it should be stored
        in Fortran array conventions.

        On exit, 'b' is overwritten with the result 'x'.  Note that it
        is the caller's responsibility to check that 'b' is big enough
        to hold the result.
        """
        bl = self.block()
        if isinstance(b, CVector):
            base,inc = b.base_and_inc()
            if inc == 1:
                b = CMatrix(b.n, 1, base, transposed=True)
                tmp = b
            else:
                raise NotImplementedError()
        elif isinstance(b, CMatrix):
            if b.transposed:
                tmp = b
            else:
                tmp = bl.define_matrix(b.m, b.n, init=b, transposed=True)
        else:
            raise TypeError("invalid argument type '%s' for rhs"%type(b))
        if not isinstance(A, CMatrix):
            raise TypeError("invalid argument type '%s' for matrix"%type(A))

        if tmp.m < A.m:
            tmpl = "not enough columns for rhs (have %d, need %d)"
            raise ValueError(tmpl%(tmp.m, A.m))
        if tmp.m < A.n:
            tmpl = "not enough columns for store result (have %d, need %d)"
            raise ValueError(tmpl%(tmp.m, A.n))

        A = bl.define_matrix(A.m, A.n, init=A, transposed=True)
        lwork = bl.define_double()
        bl.funcall(bl.lapack("dgels"), "'N'", A.m, A.n, tmp.n, A, A.m,
                   tmp, tmp.m, "&"+str(lwork), -1)
        lwork = bl.define_long(name="lwork", init=lwork+0.5)
        work = bl.define_vector(lwork, name="work")
        bl.funcall(bl.lapack("dgels"), "'N'", A.m, A.n, tmp.n, A, A.m,
                   tmp, tmp.m, work, lwork)

        if tmp != b:
            bl.store_matrix(b, tmp)

    def eigenvalues_matrix(self, A, re, im):
        """Emit code to compute the eigenvalues of a general matrix A."""
        n = A.n
        assert A.m == n and re.n == n and im.n == n
        bl = self.block()

        A = bl.define_matrix(n, n, A, name="Atmp")

        ilo = bl.define_long ("ilo")
        ihi = bl.define_long ("ihi")
        scale = bl.define_vector(n, name="scale")
        bl.funcall (bl.lapack("dgebal"), "'B'", n, A, n, "&ilo", "&ihi",
                    scale)

        tau = bl.define_vector(n-1, name="tau")
        tmp = bl.define_double ("tmp")
        bl.funcall (bl.lapack("dgehrd"), n, ilo, ihi, A, n, tau, "&tmp", -1)
        lwork = bl.define_long ("lwork", tmp+0.5)
        work = bl.define_vector(lwork, name="work")
        bl.funcall (bl.lapack("dgehrd"), n, ilo, ihi, A, n, tau, work, lwork)

        re_name,re_inc = re.base_and_inc()
        assert re_inc == 1
        im_name,im_inc = im.base_and_inc()
        assert im_inc == 1
        bl.add_include("<stdlib.h>")
        bl.funcall (bl.lapack("dhseqr"), "'E'", "'N'", n, ilo, ihi, A,
                    n, re_name, im_name, "NULL", n, work, lwork)

    def eigenvalues_indexed_symmetric_matrix(self, A, i0, i1, res=None):
        """Emit Code to get eigenvalues of a symmetric matrix.

        'A' must be a aquare matrix.  The matrix is assumed to be
        symmetric and only the entries on and above the diagonal are
        accessed.

        The function generates code to compute the eigenvalues i0,
        ..., i1-1 of A.  The eigenvalues are stored in the vector 'res' in
        ascending order (since A is symmetric, all eigenvalues are
        real).
        """
        if A.m != A.n:
            raise ValueError("matrix A is non-square")
        n = A.n
        if res is None:
            res = self.define_vector(n)
        bl = self.block()

        AB = bl.define_matrix(n, n, A, transposed=True, name="AB")
        tmpi = bl.define_long("tmpi")
        abstol = bl.define_double("abstol")
        bl.funcall(bl.lapack("dlamch"), "'S'", res=abstol)
        work = bl.define_vector(8*n, name="work")
        bl.add_definition("iwork", "long", len=5*n)
        iwork = value("iwork")

        re_name,re_inc = res.base_and_inc()
        assert re_inc == 1
        bl.add_include("<stdlib.h>")
        bl.funcall(bl.lapack("dsyevx"), "'N'", "'I'", "'U'", n,
                   AB, AB.m, 0.0, 0.0, i0+1, i1, 2*abstol, "&"+str(tmpi),
                   re_name, "NULL", 1, work, work.n, iwork, "NULL")
        return res[0:i1-i0]

    def eigenvalues_symmetric_mgb(self, A, re=None):
        """Emit Code to get the eigenvalues of a symmetric, banded matrix.

        'A' must be a matrix of class Mgb.  The matrix is assumed to
        be symmetric and only the entries on and above the diagonal
        are accessed.  The eigenvalues are stored in the vector 're'
        in ascending order.

        Since A is symmetric, all eigenvalues are real.
        """
        if A.m != A.n:
            raise ValueError("matrix A is non-square")
        n = A.n
        kd = A.ku
        if re is None:
            re = self.define_vector(n)
        assert re.n == n
        bl = self.block()
        AB = bl.define_matrix(kd+1, n, A.data[A.extra:A.extra+kd+1,:],
                              transposed=True, name="AB")
        work = bl.define_vector(3*n-2, name="work")

        re_name,re_inc = re.base_and_inc()
        assert re_inc == 1
        bl.add_include("<stdlib.h>")
        bl.funcall(bl.lapack("dsbev"), "'N'", "'U'", n, A.ku, AB, AB.m,
                   re_name, "NULL", 1, work)
        return re

    def eigenvalues_indexed_symmetric_mgb(self, A, i0, i1, res=None):
        """Emit Code to get eigenvalues of a symmetric, banded matrix.

        'A' must be a matrix of class Mgb.  The matrix is assumed to
        be symmetric and only the entries on and above the diagonal
        are accessed.  The eigenvalues are stored in the vector 'res'
        in ascending order.

        Since A is symmetric, all eigenvalues are real.
        """
        if A.m != A.n:
            raise ValueError("matrix A is non-square")
        n = A.n
        if res is None:
            res = self.define_vector(n)
        kd = A.ku
        bl = self.block()
        AB = bl.define_matrix(kd+1, n, A.data[A.extra:A.extra+kd+1,:],
                              transposed=True, name="AB")
        tmpi = bl.define_long("tmpi")
        abstol = bl.define_double("abstol")
        bl.funcall(bl.lapack("dlamch"), "'S'", res=abstol)
        work = bl.define_vector(7*n, name="work")
        bl.add_definition("iwork", "long", len=5*n)
        iwork = value("iwork")

        re_name,re_inc = res.base_and_inc()
        assert re_inc == 1
        bl.add_include("<stdlib.h>")
        bl.funcall(bl.lapack("dsbevx"), "'N'", "'I'", "'U'", n, A.ku,
                   AB, AB.m, "NULL", 0, 0, 0, i0+1, i1, 2*abstol,
                   "&"+str(tmpi), re_name, "NULL", 1, work, iwork, "NULL")
        return res[0:i1-i0]

    def eigenvalues_indexed_symmetric_mgt(self, A, i0, i1, res=None):
        """Emit Code to get eigenvalues of a symmetric, tridiagonal matrix.

        'A' must be a matrix of class Mgt.  The matrix is assumed to
        be symmetric and only the entries on and above the diagonal
        are accessed.  The eigenvalues are stored in the vector 'res'
        in ascending order.

        Since A is symmetric, all eigenvalues are real.
        """
        if A.m != A.n:
            raise ValueError("matrix A is non-square")
        n = A.n
        if res is None:
            res = self.define_vector(n)
        bl = self.block()
        D = bl.define_vector(n, A.datam, name="D")
        E = bl.define_vector(n, name="E")
        bl.store_vector(E[0:n-1], A.datar)
        abstol = bl.define_double(name="abstol")
        bl.funcall(bl.lapack("dlamch"), "'S'", res=abstol)
        m = bl.define_long(name="m")
        work = bl.define_vector(5*n, name="work")
        bl.add_definition("iwork", "long", len=5*n)
        iwork = value("iwork")
        re_name,re_inc = res.base_and_inc()
        if re_inc != 1:
            raise ValueError("vector res has increment different from 1")
        bl.add_include("<stdlib.h>")
        bl.funcall(bl.lapack("dstevx"), "'N'", "'I'", n,
                   D, E, 0, 0, i0+1, i1, 2*abstol,
                   "&"+str(m), re_name, "NULL", 1, work, iwork, "NULL")
        return res[0:i1-i0]

    def sylvester_matrix(self, A, B, C):
        """Solve the sylvester matrix equation A*X + X*B == C.

        A and B must be square matrices.

        On exit, A and B are overwritten with their respective Schur
        factorisations.  C is overwritten with the result.
        """
        if C.transposed:
            # Fortran is happy with our matrix C as is
            Atrans = A.transposed
            Btrans = B.transposed
        else:
            # solve the transposed equation B'*X' + X'*A' == C' instead
            A,B = B,A
            C = CMatrix(C.n, C.m, C.base, not C.transposed)
            Atrans = not A.transposed
            Btrans = not B.transposed

        m = A.m
        n = B.n
        assert A.m == A.n and B.m == B.n
        assert C.m == m and C.n == n

        bl = self.block()
        sdim = bl.define_long("sdim")
        wr = bl.define_vector(m, name="wl")
        wi = bl.define_vector(m, name="wi")
        U = bl.define_matrix(m, m, transposed=True, name="U")

        bl.add_include("<stdlib.h>")
        tmp = bl.define_double("tmp")
        bl.funcall(bl.lapack("dgees"),
                   "'V'", "'N'", "NULL", m, A, m, "&"+str(sdim), wr, wi,
                   U, m, "&"+str(tmp), -1, "NULL")
        lwork = bl.define_long ("lwork", tmp+0.5)
        work = bl.define_vector(lwork, name="work")
        bl.funcall(bl.lapack("dgees"),
                   "'V'", "'N'", "NULL", m, A, m, "&"+str(sdim), wr, wi,
                   U, m, work, lwork, "NULL")

        V = bl.define_matrix(n, n, transposed=True, name="V")
        bl.funcall(bl.lapack("dgees"),
                   "'V'", "'N'", "NULL", n, B, n, "&"+str(sdim), wr, wi,
                   V, n, work, lwork, "NULL")

        Ctmp = bl.define_matrix(m, n, name="Ctmp")
        bl.fmadd_matrix_matrix(1, U.trans(), C, 0, Ctmp)
        bl.fmadd_matrix_matrix(1, Ctmp, V, 0, C)

        Atrans = Atrans and "'N'" or "'T'"
        Btrans = Btrans and "'N'" or "'T'"

        scale = bl.define_double("scale")
        bl.funcall(bl.lapack("dtrsyl"), Atrans, Btrans, +1, m, n,
                   A, m, B, n, C, m, "&"+str(scale))
        bl.fmadd_matrix_matrix(1, U, C, 0, Ctmp)
        bl.fmadd_matrix_matrix(1, Ctmp, V.trans(), 0, C)
        bl.cassert("%s == 1.0"%scale)

    def lyapunov_matrix(self, A, C):
        """Solve the Lyapunov matrix equation A*X+X*A^T=C.

        A must be a square matrix, C must be symmetric.
        This method is a special case of Block.sylvester_matrix().

        On exit, A is overwritten with its Schur factorisations and C
        is overwritten with the result.
        """
        n = A.m
        assert A.n == n
        assert C.m == n and C.n == n
        bl = self.block()

        Atrans = A.transposed
        Btrans = not A.transposed

        sdim = bl.define_long("sdim")
        wr = bl.define_vector(n, name="wl")
        wi = bl.define_vector(n, name="wi")
        U = bl.define_matrix(n, n, transposed=True, name="U")

        bl.add_include("<stdlib.h>")
        tmp = bl.define_double("tmp")
        bl.funcall(bl.lapack("dgees"),
                   "'V'", "'N'", "NULL", n, A, n, "&"+str(sdim), wr, wi,
                   U, n, "&"+str(tmp), -1, "NULL")
        lwork = bl.define_long ("lwork", tmp+0.5)
        work = bl.define_vector(lwork, name="work")
        bl.funcall(bl.lapack("dgees"),
                   "'V'", "'N'", "NULL", n, A, n, "&"+str(sdim), wr, wi,
                   U, n, work, lwork, "NULL")

        Ctmp = bl.define_matrix(n, n, name="Ctmp")
        bl.fmadd_matrix_matrix(1, U.trans(), C, 0, Ctmp)
        bl.fmadd_matrix_matrix(1, Ctmp, U, 0, C)

        Atrans = Atrans and "'N'" or "'T'"
        Btrans = Btrans and "'N'" or "'T'"

        scale = bl.define_double("scale")
        bl.funcall(bl.lapack("dtrsyl"), Atrans, Btrans, +1, n, n,
                   A, n, A, n, C, n, "&"+str(scale))
        bl.fmadd_matrix_matrix(1, U, C, 0, Ctmp)
        bl.fmadd_matrix_matrix(1, Ctmp, U.trans(), 0, C)
        bl.cassert("%s == 1.0"%scale)

    def constant_pi(self):
        "Return a Value representing the constant pi in the generated program."
        self.add_include ('<math.h>')
        return  value("M_PI")

    def fabs(self, x):
        "Return a Value representing 'fabs(x)' in the generated program."
        self.add_include ('<math.h>')
        return  value("fabs(%s)"%x)

    def sqrt(self, x):
        "Return a Value representing 'sqrt(x)' in the generated program."
        self.add_include ('<math.h>')
        return  value("sqrt(%s)"%x)

    def exp(self, x):
        "Return a Value representing 'exp(x)' in the generated program."
        self.add_include('<math.h>')
        if isinstance(x, Vector):
            return FnVector(x, lambda a: "exp(%s)"%a)
        if isinstance(x, Matrix):
            return FnMatrix(x, lambda a: "exp(%s)"%a)
        return value("exp(%s)"%x)

    def pow10(self, x):
        "Return a Value representing 'pow10(x)' in the generated program."
        self.add_include('<math.h>')
        return value("pow(10, %s)"%x)

    def log(self, x):
        "Return a Value representing 'log(x)' in the generated program."
        self.add_include('<math.h>')
        if isinstance(x, Vector):
            return FnVector(x, lambda a: "log(%s)"%a)
        if isinstance(x, Matrix):
            return FnMatrix(x, lambda a: "log(%s)"%a)
        return value("log(%s)"%x)

    def log10(self, x):
        "Return a Value representing 'log10(x)' in the generated program."
        self.add_include('<math.h>')
        if isinstance(x, Vector):
            return FnVector(x, lambda a: "log10(%s)"%a)
        if isinstance(x, Matrix):
            return FnMatrix(x, lambda a: "log10(%s)"%a)
        return value("log10(%s)"%x)

    def sin(self, x):
        "Return a Value representing 'sin(x)' in the generated program."
        self.add_include ('<math.h>')
        # TODO: the result it not really const_C (workaround for xxl.py)
        return  value("sin(%s)"%x, True)

    def cos(self, x):
        "Return a Value representing 'cos(x)' in the generated program."
        self.add_include ('<math.h>')
        # TODO: the result it not really const_C (workaround for xxl.py)
        return  value("cos(%s)"%x, True)

    def sort_vector(self, x, reverse=False):
        xbase,xinc = x.base_and_inc ()
        assert xinc == 1
        self.add_include ('<stdlib.h>')
        if reverse:
            self.funcall ('qsort', xbase, x.n, "sizeof(double)",
                          self._auxfn("compare_doubles_reverse"))
        else:
            self.funcall ('qsort', xbase, x.n, "sizeof(double)",
                          self._auxfn("compare_doubles"))

    def _output_open(self, fname=None, fd=None, force_block=False, mode='w'):
        if force_block or fname is not None:
            bl = self.block()
        else:
            bl = self

        bl.add_include ('<stdio.h>')
        if fname is not None:
            fd = bl.define_file("fd", fname, mode)
            bl.destructors.append("fclose(%s);"%fd)
            cprintf = [ 'fprintf', fd ]
            cputchar = "fputc('%s', "+str(fd)+")"
        elif fd is not None:
            cprintf = [ 'fprintf', fd ]
            cputchar = "fputc('%s', "+str(fd)+")"
        else:
            fd = value("stdout")
            cprintf = ['printf']
            cputchar = "putchar('%s')"

        return  bl, fd, cprintf, cputchar

    def output_int(self, *args, **kw):
        if len(args) == 0: return

        fname = kw.get("fname", None)
        fd = kw.get("fd", None)
        fmt = kw.get("fmt", "%d")
        flush = kw.get("flush", False)

        bl, fd, cprintf, cputchar = self._output_open(fname, fd)
        cprintf.append(r'"%s\n"'%" ".join([fmt]*len(args)))
        bl.funcall (*(cprintf+list(args)))
        if flush:
            bl.add_include("<stdlib.h>")
            bl.funcall("fflush", fd)

    def output_double(self, *args, **kw):
        if len(args) == 0: return

        fname = kw.get("fname", None)
        fd = kw.get("fd", None)
        fmt = kw.get("fmt", "%f")
        flush = kw.get("flush", False)
        mode = kw.get("mode", 'w')

        bl, fd, cprintf, cputchar = self._output_open(fname, fd, mode=mode)
        cprintf.append(r'"%s\n"'%" ".join([fmt]*len(args)))
        bl.funcall (*(cprintf+list(args)))
        if flush:
            bl.add_include("<stdlib.h>")
            bl.funcall("fflush", fd)

    def output_vector(self, *args, **kw):
        """Output one or more vectors.

        If the optional keyword argument 'fname' is given, it is used
        as the name of an output file.  The optional keyword argument
        'mode' can be set to 'a' to append to this file, otherwise it
        is overwritten.  If 'fname' is not set but the optional
        keyword argument 'fd' is given, it is used as a file
        descriptor for the output.  Otherwise stdout is used.

        If the optional keyword argument 'flush' is True, flush the
        output after writing the vectors.

        The remaining arguments must be vectors of equal length which
        form the columns of the output.
        """
        if len(args) == 0: return

        fname = kw.get("fname", None)
        fd = kw.get("fd", None)
        fmt = kw.get("fmt", "%f")
        mode = kw.get("mode", 'w')
        flush = kw.get("flush", False)

        bl, fd, cprintf, cputchar = self._output_open(fname, fd, False, mode)
        cprintf.append(r'"%s\n"'%" ".join([fmt]*len(args)))

        n = args[0].n
        keys = set()
        for x in args:
            if x.n != n:
                tmpl = "vectors have different lengths (%s vs. %s)"
                raise ValueError(tmpl%(x.n,n))
            keys |= x.special()
        if value(1) in keys:
            keys.add(value(0))
        if n-2 in keys:
            keys.add(n-1)

        done = set()
        a = value(0)
        while a in keys:
            bl.funcall (*(cprintf + [str(x[a]) for x in args]))
            done.add (a)
            a += 1

        tail = []
        b = n-1
        while b in keys:
            if b < a: break
            tail.append(b)
            done.add (b)
            b -= 1

        if not b < a:
            todo = keys - done
            if len(todo)>0:
                raise NotImplementedError()
            i = bl._loop_index(0)
            loop = bl.loop (i, a, b+1)
            loop.funcall (*(cprintf + [str(x[i]) for x in args]))

        for b in reversed(tail):
            bl.funcall (*(cprintf + [str(x[b]) for x in args]))

        if flush:
            bl.add_include("<stdlib.h>")
            bl.funcall("fflush", fd)

    def output_matrix(self, A, fname=None, fd=None, fmt="%f", columns=79,
                      mode="w"):
        """Output a matrix.

        If 'fname' is not None, it is used as the name of an output
        file.  Otherwise, if 'fd' is not None, it is used as a file
        descriptor for the output.  Otherwise stdout is used.

        'fmt' is a printf format string.  If is contains a '*', the
        generated code will try hard to preserve columns in the output
        and to fit the output into 'columns' screen columns.
        """
        fancy = "*" in fmt
        bl, _, cprintf, cputchar = self._output_open(fname, fd, fancy, mode)
        if fancy:
            states = ["try","output"]
        else:
            states = ["plain"]
        fmt = '"%s"'%fmt

        m = A.m.simplify()
        n = A.n.simplify()

        for state in states:
            if m == 0 or n == 0: return

            if state=="try":
                cw = bl.define_int(columns/(n+1)-1, name="cw")
                bl.statement ("if (cw<3)  cw = 3;")
                mw = bl.define_int(0, name="mw")
                buf = bl.define_string (cw+1, name="buffer")

            if m == 1:
                l1 = bl
                i = 0
            else:
                i = bl._loop_index(0)
                l1 = bl.loop(i, 0, m)

            if n == 1:
                l2 = l1
                j = 0
            else:
                j = l1._loop_index(1)
                l2 = l1.loop(j, 0, n)
                if state!="try":
                    l2.cond("%s>0"%j, cputchar%" "+";")

            val = str(A.get(i,j,False))
            if state=="try":
                l2.funcall ("snprintf", buf, cw+1, fmt, 0, val)
                w = l2.define_int(name="w")
                bl.add_include ('<string.h>')
                l2.funcall ("strlen", buf, res=w)
                l2.statement ("if (w>mw)  mw = w;")
                bl.statement ("if (cw>mw+2)  cw = mw+2;")
            elif state=="output":
                extra = l1.define_int(0, name="extra")
                tw = l2.define_int(cw-extra, name="tw")
                l = l2.define_int(name="len")
                args = cprintf + [fmt, "tw>1?tw:1", val]
                l2.funcall (*args, **{"res":l})
                l2.store_scalar (extra, l-tw)
                l1.statement (cputchar%'\\n'+";")
            else:
                args = cprintf + [fmt, val]
                l2.funcall (*args)
                l1.statement (cputchar%'\\n'+";")

    def output_string(self, *args, **kw):
        """Output a string, optionally substitute additional arguments.

        This function is just a wrapper for the C function ``printf``.
        The following keyword arguments are recognised:

          * fname=...: create a file with the given name for output
          * fd=...: write into the given file descriptor
          * flush=...: if true, disable buffering of output

        Beware of format string vulnerabilities when using this
        method.  The correct way to ouput a string is as follows::

            block.output_string("%s", "some text")

        If neither the `fname` nor the `fd` argument are given,
        generate code to print the output to ``stdout``.
        """
        if len(args) == 0:
            return

        fname = kw.get("fname", None)
        fd = kw.get("fd", None)
        flush = kw.get("flush", False)

        def maybe_quote(s):
            if isinstance(s, basestring):
                return string_to_C(s)
            else:
                return s

        bl, fd, cprintf, cputchar = self._output_open(fname, fd)
        bl.funcall(*(cprintf+[ maybe_quote(x) for x in args ]))
        if flush:
            bl.funcall("fflush", fd)

    def read_vector(self, x, fname):
        bl = self.block()
        bl.add_include ('<stdio.h>')
        bl.add_include ('<assert.h>')
        bl.add_definition ("fp", "FILE", 'fopen (%s, "r")'%fname, 0)
        i = self._loop_index(0)
        l1 = bl.loop (i, 0, x.n)
        tmp = l1.define_double ("tmp")
        rc = l1.define_int(name="rc")
        l1.funcall ("fscanf", "fp", '"%lf"', "&tmp", res=rc)
        l1.statement ("assert (rc == 1);")
        l1.store_scalar (x[i], tmp)
        bl.statement ("fclose (fp);")

    def read_matrix(self, A, fname, fcol=None):
        """Read matrix entries from a file.

        'A' must be an existing matrix, 'fname' is the file name as a
        C expression (e.g. fname='"file.dat"').  If 'fcol' is set, it
        must be the number of columns in the input file, the columns
        which do not fit into the matrix A are ignored.  Otherwise the
        number of columns of the matrix A and of the input file must
        coincide.
        """
        if not fcol: fcol = A.n
        bl = self.block()
        self.add_include ('<stdio.h>')
        self.add_include ('<assert.h>')
        bl.add_definition ("fp", "FILE", 'fopen (%s, "r")'%fname, 0)
        i = self._loop_index(0)
        j = self._loop_index(1)
        l1 = bl.loop (i, 0, A.m)
        l2 = l1.loop (j, 0, fcol)
        x = l2.define_double ("x")
        rc = l2.define_int(name="rc")
        l2.funcall ("fscanf", "fp", '"%lf"', "&x", res=rc)
        l2.statement ("assert (rc == 1);")
        if fcol == A.n:
            l2.store_scalar (A[i,j], x)
        else:
            cond = l2.cond ("%s < %s"%(j,A.n))
            cond.store_scalar (A[i,j],x)
        bl.statement ("fclose (fp);")

class MaybeBracedBlock(Block):

    def __init__(self, parent, cmd, indent=0):
        Block.__init__ (self, parent, indent, None, None)
        self.cmd = cmd

    def lines(self, pfx=None):
        if pfx is None: pfx = "  "*self.indent
        ll = [ l for l in Block.lines (self, pfx="") ]
        if len(ll) == 1:
            l = ll[0]
            s = pfx + self.cmd + l
            if len(s)<80:
                yield s
            else:
                yield pfx + self.cmd + " {"
                yield pfx + l
                yield pfx + "}"
        else:
            yield pfx + self.cmd + " {"
            for l in ll: yield pfx + l
            yield pfx + "}"

class Function(Block):

    """Represent user-defined C functions.

    This class represents user-defined functions in a C program.
    Objects of this class are normally created using the
    Source.function() class method.
    """

    def __init__(self, source, name, *args, **kwargs):
        kwargs = checkOptArgs(kwargs,
                              rtype="void",
                              static=False,
                              headerfd=None)
        rtype = kwargs["rtype"]
        static = kwargs["static"]
        headerfd = kwargs["headerfd"]
        if len(args) == 0:
            aa = ["void"]
        else:
            aa = []
            for arg, type, n in args:
                if n is None:
                    aa.append(type+" "+arg)
                elif n == 0:
                    aa.append(type+" *"+arg)
                else:
                    aa.append(type+" "+arg+"[%s]"%n)
            argstr = ", ".join(aa)
        if static:
            lhs = "static "+rtype
        else:
            lhs = rtype
        defn = lhs+"\n"+"\n".join(break_list(aa, name+"(", ")"))
        Block.__init__(self, source, first=defn+"\n{")

        if not static and headerfd is not None:
            pre = "extern "+rtype+" "+name+"("
            decl = "\n".join(break_list(aa, pre, ");"))
            headerfd.write(decl+"\n")

        self.args = []
        for arg, type, n in args:
            x = value(arg)
            self.symbols[arg] = x
            self.args.append(x)

class Main(Block):

    """Represent the C 'main' function.

    This class represents the 'main' function in a C program.  Objects
    of this class are normally created using the :meth:`Source.main()`
    class method.
    """

    def __init__(self, source):
        Block.__init__ (self, source,
                        first="int\nmain(int argc, char **argv)\n{",
                        position="tail")
        self.ret = "return 0;"
        self.next_arg = 1
        self.optional_args = False

    def get_arg_int(self, default=None):
        """Convert the next command line argument to int and return it.

        If 'default' is set and the argument is not given on the command
        line, return 'default' instead.
        """
        self.add_include ('<stdlib.h>')
        if default is None:
            if self.optional_args:
                msg = "non-optional argument after optional argument"
                raise TypeError(msg)
            res = value ("atoi (argv[%d])"%self.next_arg)
        else:
            tmpl = "(argc>%s) ? atoi (argv[%d]) : %s"
            res = tmpl%(self.next_arg, self.next_arg, default)
            self.optional_args = True
        self.next_arg += 1
        return  res

    def get_arg_double(self, default=None):
        """Convert the next command line argument to double and return it.

        If 'default' is set and the argument is not given on the command
        line, return 'default' instead.
        """
        self.add_include ('<stdlib.h>')
        if default is None:
            if self.optional_args:
                msg = "non-optional argument after optional argument"
                raise TypeError(msg)
            res = value ("atof (argv[%d])"%self.next_arg)
        else:
            tmpl = "(argc>%s) ? atof (argv[%d]) : %s"
            res = tmpl%(self.next_arg, self.next_arg, default)
            self.optional_args = True
        self.next_arg += 1
        return  res

    def get_arg_string(self, default=None):
        """Return the next command line argument as a string.

        If 'default' is set and the argument is not given on the command
        line, return 'default' instead.
        """
        if default is None:
            if self.optional_args:
                msg = "non-optional argument after optional argument"
                raise TypeError(msg)
            res = value ("argv[%d]"%self.next_arg)
        else:
            tmpl = "(argc>%s) ? argv[%d] : %s"
            res = tmpl%(self.next_arg, self.next_arg, default)
            self.optional_args = True
        self.next_arg += 1
        return  res

class Sniplet(Code):

    """Copy a bit of code from an external file."""

    def __init__(self, fname, category=None):
        Code.__init__ (self)
        if category:
            fname=os.path.join(DATADIR,category,fname)
        f=open(fname)
        for l in f.readlines():
            self.body.append(l.rstrip())
        f.close()

class SourceComment(Code):

    """Leading comment block of a C source file."""

    def __init__ (self, fname, desc="", author_name=None, author_email=None,
                  license=None):
        Code.__init__(self)
        self.add_head ("/* "+fname+" - "+desc)

        if not author_name:
            author_name=_default_author_name
        if not author_email:
            author_email=_default_author_email
        if author_name:
            year=time.strftime("%Y")
            s=" * Copyright (C) %s  %s"%(year,author_name)
            if author_email:
                s+=" "+author_email
            self.add_head (" *")
            self.add_head (s)

        if license:
            self.add(Sniplet(license, "licenses"))

        self.add_tail (" */")

class SourceIncludes(Code):

    """Keep track of required C preprocessor #include statements."""

    def __init__(self):
        Code.__init__(self)
        self.body = []

    def add(self, header):
        if header not in self.body: self.body.append (header)

    def lines(self):
        done = False
        for h in self.body:
            if h[0] == "<":
                yield "#include "+h
                done = True

        if done: yield ""
        for h in self.body:
            if h[0] != "<":
                yield "#include "+h

class SourceDefines(Code):

    """C preprocessor #define statements near the top of a C source file."""

    def __init__(self):
        Code.__init__(self)
        self.defines = []

    def add_define(self, name, val):
        x = value (name, known_c=True)
        self.add ("#define "+name+" "+value(val).protect(4))
        return x

class Source(Code):

    """Python representation of a C source file.

    Each object of this class represents a complete C source file.
    Initially the file file only consists of a leading comment.  C
    Functions can be added using the :meth:`function()` and
    :meth:`main()` methods.  Once the C source file is constructed
    completely, it can written to disk using :meth:`save()`.
    """

    def __init__ (self, fname=None, desc="", author_name=None,
                  author_email=None, license=None, lapack_headers=False):
        Code.__init__(self, seplines=1)

        if not fname:
            from inspect import currentframe, getfile
            frame = currentframe()
            try:
                while frame.f_back:
                    frame = frame.f_back
                base,ext = os.path.splitext(os.path.basename(getfile(frame)))
                fname = base+".c"
            finally:
                del frame

        self.progname = fname
        self.auxfns = get_auxfns()
        self.auxfn_present = {}

        self.comment = SourceComment(fname, desc,
                                     author_name, author_email, license)
        self.add(self.comment)
        self.includes = SourceIncludes()
        self.add(self.includes)
        self.defines = SourceDefines()
        self.add(self.defines)

        self.lapack_headers = lapack_headers

    def _auxfn(self, name, caller_id):
        if name in self.auxfn_present:
            aux_id = self.auxfn_present[name]
        else:
            aux_id = self.auxfns[name](self).id
            self.auxfn_present[name] = aux_id
        self.depends.add((caller_id,aux_id))
        return name

    def lapack(self, name, caller_id):
        if name in self.auxfn_present:
            aux_id = self.auxfn_present[name]
        else:
            if self.lapack_headers:
                self.add_include ('"%s.h"'%name)
                aux_id = self.includes.id
            else:
                fn = Sniplet(name+".inc", "LAPACK")
                self.add(fn)
                aux_id = fn.id
            self.auxfn_present[name] = aux_id
        self.depends.add((caller_id,aux_id))
        return name

    def add_include(self, header):
        self.includes.add(header)

    def add_define(self, name, val):
        return  self.defines.add_define(name, val)

    def function(self, name, *args, **kwargs):
        """Add a new C function to the Source.

        Create a new, empty C function and place it before the 'main'
        function in the source file.  'name' is the function name, all
        other regular arguments are C function arguments (see below).
        The keyword argument 'rtype' gives the return type as a
        string.

        Each C function argument is represented by a tuple
        '(name,type,len)' where 'name' is the argument name and 'type'
        is the argument type as a string.  If 'len' is 0, the argument
        is a pointer to the specified type; if 'len' is None, the
        argument has the given type, otherwise it is declared as an
        array of length 'len'.

        The return value is a function object (of class Block).  The
        class methods of this object can be used to add statements to
        the new function.
        """
        fn = Function(self, name, *args, **kwargs)
        return fn

    def main(self):
        """Add a ``main()`` function to the Source.

        This is a specialised version of :meth:`function`, which adds
        a standard main function to the source.  The return value
        is an object of class :class:`Main`.
        """
        return  Main(self)

    def save(self, fname=None):
        """Save the C source file to disk.

        The optinal argument `fname`, if given, overwrites the file
        name set by the corresponding argument to the :class:`Source`
        constructor.  If no filename is given, the source is printed
        to stdout.
        """
        if fname is None:
            fname = self.progname
        if fname and fname != "-":
            fd = open (fname, "w")
            for l in self.lines():
                try:
                    fd.write (l+"\n")
                except:
                    print "error: "+str(l)
                    raise
        else:
            for l in self.lines(): print l
