# This file is part of the CSimGen package.
#
# Copyright 2007, 2009  Jochen Voss <voss@seehuhn.de>
#
# CSimGen is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301 USA

"""Helpers for text formatting.

This module provides helper functions to format the text in the
generated C source files.
"""

def split_it(args, padding="", start1="", start2=None, sep=", ",
             end1=",", end2="", linebreak=None, maxwidth=79):
    """A generator to format args and split into lines.

    The elements of the list 'args' are converted into strings and
    grouped into lines.  'args' must not be empty.

    The first line is preceeded by 'padding+start1', all subsequent
    lines are preceeded by 'padding+start2' (or by 'padding+start1',
    if 'start2' is None).  The elements within a line are separated by
    'sep'.  All lines except for the last are terminated by 'end1',
    the last line is terminated by 'end2'.  If possible, lines are at
    most 'maxwidth' characters long.
    """
    if start2 is None:
        start2 = start1
    args = [ str(arg) for arg in args[:-1] ] + [ str(args[-1])+end2 ]

    line = padding + start1 + args.pop(0)
    while args:
        test = sep.join([line]+args)
        if len(test) <= maxwidth and linebreak not in args[:-1]:
            line = test
            break

        arg = args.pop(0)
        if args and arg == linebreak:
            yield line + end1
            line = padding + start2 + args.pop(0)
            continue
        if len(line+sep+arg+end1) > maxwidth:
            yield line+end1
            line = padding + start2 + arg
        else:
            line += sep+arg
    yield line

def split_list(args, pre="", post="", sep=", ", maxwidth=79):
    if not args: return
    for l in split_it(args, padding=pre, sep=sep, end1=post, end2=post,
                      maxwidth=maxwidth):
        yield l

def break_list(args, pre="", post="", sep1=",", sep2=" ", maxwidth=79):
    """A generator which returns the lines of pre+args+post."""
    if not args:
        s = pre+post
        if s:
            yield s
        return
    for l in split_it(args, padding="", start1=pre,
                      start2=" "*len(pre), sep=sep1+sep2, end1=sep1,
                      end2=post, maxwidth=maxwidth):
        yield l

def string_to_C(s):
    res = [ '"' ]
    for c in s:
        if c == '\n':
            res.append(r'\n')
        elif c == '\t':
            res.append(r'\t')
        elif c == '\r':
            res.append(r'\r')
        elif c == '\v':
            res.append(r'\v')
        elif c == '\\':
            res.append(r'\\')
        elif c == '"':
            res.append(r'\"')
        elif ord(c) >= 32 and ord(c) < 127:
            res.append(c)
        else:
            res.append(r'\%03o'%ord(c))
    res.append('"')
    return "".join(res)
