#! /usr/bin/env python
#
# Copyright 2007  Jochen Voss <voss@seehuhn.de>
#
# This file is part of the CSimGen package.
#
# CSimGen is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301 USA

"""Matrix representation inside the code generator.

This module implements a Matrix class.  The matrices are matrices in
the python code generator, the elements of the matrices are Values.
The sub-class 'CMatrix' is used to represent matrices from the
generated C source inside the code generator.

Note that functions of this module never generate C code.  Assignments
to matrix elements change the matrix represented in the code
generator, they do not change matrix elements in the generated code
(use instances of CodeGen to do the latter).

WARNING: this is an experimental version of CSimGen.  The API is not
yet finalised and may change without further notice.
"""

__author__ = 'Jochen Voss'
__version__ = '0.1'

from value import value, Value
from vector import _slicify, FnVector, ConstVector, ComputedVector, CVector

class Matrix (object):

    def __init__(self, m, n):
        self.m = value(m)
        self.n = value(n)
        self.items = {}

    def __getitem__(self, idx):
        i, j = idx
        if isinstance (i, slice) or isinstance (j, slice):
            i0, i1, istep = _slicify (i,0,self.m)
            j0, j1, jstep = _slicify (j,0,self.n)
            return  SubMatrix (self, i0, i1, istep, j0, j1, jstep)
        i = value(i)
        j = value(j)
        if i<0 or i>=self.m or j<0 or j>=self.n:
            raise IndexError()
        if (i,j) in self.items:
            return  self.items[(i,j)]
        else:
            return  self.get (i, j)

    def __setitem__(self, idx, val):
        i, j = idx
        i = value(i)
        j = value(j)
        if i<0 or i>=self.m or j<0 or j>=self.n:
            raise IndexError()
        val = value(val)
        if hasattr (self, "set"):
            self.set (i, j, val)
        else:
            self.items[(i,j)] = val

    def __pos__(self):
        return  self

    def __neg__(self):
        return  FnMatrix (self, lambda x: -x)

    def __add__(self, other):
        return  BinopMatrix (self, other, lambda x, y: x+y)

    def __sub__(self, other):
        return  BinopMatrix (self, other, lambda x, y: x-y)

    def __mul__(self, other):
        try:
            x = value(other)
        except TypeError:
            pass
        else:
            return  FnMatrix (self, lambda val: val*x)
        raise TypeError("cannot multiply Matrix and %s"%type(other))

    def __div__(self, other):
        x = value(other)
        return  FnMatrix (self, lambda val: val/x)

    def __rmul__(self, other):
        x = value(other)
        return  FnMatrix (self, lambda val: x*val)

    def trans(self):
        """Return the transpose of a matrix."""
        return  TransMatrix(self)

    def as_vector(self):
        assert  self.n == 1
        return  ComputedVector (self.m, lambda i: self[i,0])

class SubMatrix (Matrix):

    def __init__(self, A, i0, i1, istep, j0, j1, jstep):
        if i0 < 0 or i1 > A.m or j0 < 0 or j1 > A.n or istep < 1 or jstep < 1:
            raise IndexError()
        Matrix.__init__ (self, (i1-i0)/istep, (j1-j0)/jstep)
        self.A = A
        self.i0, self.i1, self.istep = map(value, [i0, i1, istep])
        self.j0, self.j1, self.jstep = map(value, [j0, j1, jstep])
        if hasattr(A, "transposed"):
            self.transposed = A.transposed

    def get(self, i, j, force_lvalue=True):
        return  self.A.get(self.i0+i*self.istep, self.j0+j*self.jstep,
                           force_lvalue)

    def set(self, i, j, val):
        self.A[self.i0+i*self.istep, self.j0+j*self.jstep] = val

class VecMatrix (Matrix):

    """Consider vectors of length n as n times 1 matrices."""

    def __init__(self, x):
        Matrix.__init__ (self, x.n, 1)
        self.x = x

    def get(self, i, j, force_lvalue=True):
        return  self.x[i]

    def set(self, i, j, val):
        self.x[i] = val

class TransMatrix(Matrix):

    """The transpose of a matrix."""

    def __init__(self, A):
        Matrix.__init__ (self, A.n, A.m)
        self.A = A

    def get(self, i, j, force_lvalue=True):
        return self.A[j,i]

class BinopMatrix (Matrix):

    def __init__(self, A, B, fn):
        m = A.m
        n = A.n
        if B.m != m or B.n != n:
            raise ValueError("matrices must have equal shape")
        Matrix.__init__ (self, m, n)
        self.A = A
        self.B = B
        self.fn = fn

    def get(self, i, j, force_lvalue=True):
        return  self.fn (self.A[i,j],self.B[i,j])

class FnMatrix (Matrix):

    def __init__(self, A, fn):
        Matrix.__init__ (self, A.m, A.n)
        self.A = A
        self.fn = fn

    def get(self, i, j, force_lvalue=True):
        return  self.fn (self.A[i,j])

class ExplicitMatrix (Matrix):

    def __init__(self, m, n, init):
        Matrix.__init__ (self, m, n)
        self.values = init

    def get(self, i, j, force_lvalue=True):
        return self.values[int(i*self.n+j)]

    def set(self, i, j, val):
        self.values[i*self.m+j] = val

class CMatrix(Matrix):

    def __init__(self, m, n, base, transposed=False):
        Matrix.__init__ (self, m, n)
        self.base = value(base).simplify()
        self.transposed = transposed

    def get(self, i, j, force_lvalue=True):
        if self.transposed:
            return value("%s[%s]"%(self.base,(j*self.m+i).simplify()))
        else:
            return value("%s[%s]"%(self.base,(i*self.n+j).simplify()))

    def trans(self):
        return CMatrix (self.n, self.m, self.base, not self.transposed)

    def spec(self):
        return (self.base, "double", self.m*self.n)

    def __str__(self):
        return  str(self.base)

    def get_column(self, k):
        """Return column k of the matrix as a vector."""
        if self.transposed:
            return CVector(self.m, self.base, k*self.m, 1)
        else:
            return CVector(self.m, self.base, k, self.n)

    def get_row(self, k):
        """Return row k of the matrix as a vector."""
        if self.transposed:
            return CVector(self.n, self.base, k, self.m)
        else:
            return CVector(self.n, self.base, k*self.n, 1)

    def get_diag(self, k=0):
        """Return the diagonal of the matrix as a vector.

        The parameter k gives the number of the diagonal.  k==0 is the
        main diagonal, k==1 is the first superdiagonal etc.
        """
        if self.transposed:
            stride = self.m
        else:
            stride = self.n

        if k >= 0:
            l = min(self.m, self.n - k)
        elif k < 0:
            l = min(self.m + k, self.n)
        else:
            raise NotImplementedError()
        if l <= 0:
            raise ValueError("invalid diagonal number %s"%k)

        if (k<0 and not self.transposed) or (k>0 and self.transposed):
            return  CVector(l, self.base, abs(k)*stride, stride+1)
        else:
            return  CVector(l, self.base, abs(k), stride+1)

class ComputedMatrix(Matrix):

    def __init__(self, m, n, fn):
        Matrix.__init__(self, m, n)
        self.fn = fn

    def get(self, i, j, force_lvalue=True):
        return self.fn(i, j)

class ConstMatrix (Matrix):

    def __init__(self, m, n, val):
        Matrix.__init__ (self, m, n)
        self.val = value(val)

    def get(self, i, j, force_lvalue=True):
        return  self.val

class DiagonalMatrix (Matrix):

    def __init__(self, n, diag):
        if isinstance(diag,(int,float,Value)):
            diag = ConstVector (n, diag)
        if diag.n != n:
            raise ValueError("dimensions don't match")
        Matrix.__init__ (self, n, n)
        self.diag = diag

    def get(self, i, j, force_lvalue=True):
        i = i.simplify()
        j = j.simplify()
        if i == j:
            return  self.diag[i]
        elif i != j:
            return  value(0.0)
        else:
            return  value("(%s==%s?%s:0)"%(i,j,self.diag[i]))

    def set(self, i, j, val):
        if i == j:
            self.diag[i] = val
        else:
            raise IndexError("attempt to set non-diagonal element")

    def inv(self):
        return DiagonalMatrix (self.n, FnVector(self.diag, lambda x: 1.0/x))

class IdentityMatrix (DiagonalMatrix):

    def __init__(self, n):
        DiagonalMatrix.__init__ (self, n, ConstVector(n,1))
