#! /usr/bin/env python
#
# Copyright 2007  Jochen Voss <voss@seehuhn.de>
#
# This file is part of the CSimGen package.
#
# CSimGen is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301 USA

"""General banded matrices for use with LAPACK.

WARNING: this is an experimental version of CSimGen.  The API is not
yet finalised and may change without further notice.
"""

__author__ = 'Jochen Voss'
__version__ = '0.1'

from value import value
from vector import ConstVector
from matrix import Matrix, CMatrix, ConstMatrix

class Mgb (Matrix):
    def __init__(self, m, n, kl, ku, data, extra=0):
        """Create a banded matrix for use with LAPACK.

        The matrix 'data' is a (kl+ku+1),n matrix which
        contains the bands in the form used by DGBMV.
        """
        assert data.m == extra+kl+ku+1 and data.n == n
        Matrix.__init__ (self, m, n)
        self.kl = kl
        self.ku = ku
        self.extra = extra
        self.data = data

    def get(self, i, j, force_lvalue=True):
        i = value(i).simplify()
        j = value(j).simplify()
        if j<i-self.kl or j>i+self.ku: return value(0.0)
        idx = self.extra+self.ku+i-j
        if force_lvalue or (j>=i-self.kl and j<=i+self.ku):
            return self.data[idx,j]
        xx = (j, i-self.kl, j, i+self.ku, self.data[idx,j])
        return  value("(%s<%s||%s>%s?0:%s)"%xx)

    def set(self, i, j, val):
        if j<i-self.kl or j>i+self.ku:
            if val == 0:  return
            raise ValueError()
        idx = self.extra+self.ku+i-j
        self.data[idx,j] = val

    def get_diag(self, k=0):
        """Return the diagonal of a matrix as a vector.

        The parameter k gives the number of the diagonal.  k==0 is the
        main diagonal, k==1 is the first superdiagonal etc.
        """
        if k >= 0:
            l = min(self.m, self.n - k)
        elif k < 0:
            l = min(self.m + k, self.n)
        else:
            raise NotImplementedError()
        if l <= 0:
            raise ValueError("invalid diagonal number %s"%k)

        if k > self.ku or k < -self.kl:
            return ConstVector(l, 0)

        if k >= 0:
            return self.data.get_row(self.extra+self.ku-k)[k:k+l]
        else:
            return self.data.get_row(self.extra+self.ku-k)[0:l]

class CMgb (Mgb):
    def __init__(self, m, n, kl, ku, name, extra=0):
        data = CMatrix (extra+kl+ku+1, n, name, transposed=True)
        Mgb.__init__ (self, m, n, kl, ku, data, extra)
        self.name = name

    def spec(self):
        return  self.data.spec()

    def __str__(self):
        return  self.name

class ConstMgb(Mgb):

    def __init__(self, m, n, kl, ku, val):
        Mgb.__init__(self, m, n, kl, ku,
                     ConstMatrix(kl+ku+1, n, val))
