#! /usr/bin/env python
#
# Copyright 2007  Jochen Voss <voss@seehuhn.de>
#
# This file is part of the CSimGen package.
#
# CSimGen is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301 USA

"""Tridiagonal matrix representation inside the code generator.

Exported classes:

    Mgt -- general tridiagonal matrices
    ConstMgt -- tridiagonal matrices where the diagonals are constant

WARNING: this is an experimental version of CSimGen.  The API is not
yet finalised and may change without further notice.
"""

__author__ = 'Jochen Voss'
__version__ = '0.1'

from value import value, Value
from vector import Vector, ConstVector, ComputedVector
from matrix import Matrix

class Mgt (Matrix):

    """Tridiagonal matrix class for use with BLAS and LAPACK."""

    def __init__(self, n, datal, datam, datar):
        if datal.n != n-1 or datam.n != n or datar.n != n-1:
            raise ValueError("dimensions of the diagonals don't match")
        Matrix.__init__ (self, n, n)
        self.datal = datal
        self.datam = datam
        self.datar = datar

    def __add__(self, other):
        if isinstance (other, Mgt):
            datal = self.datal+other.datal
            datam = self.datam+other.datam
            datar = self.datar+other.datar
            return  Mgt (self.n, datal, datam, datar)
        else:
            return  Matrix.__add__ (self, other)

    def __sub__(self, other):
        if isinstance (other, Mgt):
            datal = self.datal-other.datal
            datam = self.datam-other.datam
            datar = self.datar-other.datar
            return  Mgt (self.n, datal, datam, datar)
        else:
            return  Matrix.__sub__ (self, other)

    def __mul__(self, other):
        if isinstance (other, Vector):
            if self.n != other.n: raise ValueError("dimensions don't match")
            l = self.datal
            m = self.datam
            r = self.datar
            fn = lambda i: l[i-1]*other[i-1]+m[i]*other[i]+r[i]*other[i+1]
            N = self.m
            res = ComputedVector (N, fn)
            res[0] = m[0]*other[0]+r[0]*other[1]
            res[N-1] = l[N-2]*other[N-2]+m[N-1]*other[N-1]
            return  res
        try:
            x = value(other)
        except TypeError:
            pass
        else:
            datal = self.datal*other
            datam = self.datam*other
            datar = self.datar*other
            return  Mgt (self.n, datal, datam, datar)
        raise TypeError("cannot multiply Mgt and %s"%type(other))

    def __rmul__(self, other):
        datal = other*self.datal
        datam = other*self.datam
        datar = other*self.datar
        return  Mgt (self.n, datal, datam, datar)

    def __div__(self, other):
        datal = self.datal/other
        datam = self.datam/other
        datar = self.datar/other
        return  Mgt (self.n, datal, datam, datar)

    def get(self, i, j, force_lvalue=True):
        i = value(i).simplify()
        j = value(j).simplify()
        if j<i-1 or j>i+1: return value(0.0)
        if j==i-1: return self.datal[j]
        if j==i: return self.datam[i]
        if j==i+1: return self.datar[i]
        if force_lvalue:
            raise IndexError("invalid index (%s,%s)"%(i,j))
        xx = (j, i-1, self.datal[j],
              j, i, self.datam[i],
              j, i+1, self.datar[i])
        return  value("(%s==%s?%s:%s==%s?%s:%s==%s?%s:0)"%xx)

    def set(self, i, j, val):
        if j<i-1 or j>i+1:
            if val == 0:  return
            raise ValueError()
        if j==i-1:
            self.datal[j] = val
            return
        if j==i:
            self.datam[i] = val
            return
        if j==i+1:
            self.datar[i] = val
            return
        raise IndexError()

class ConstMgt (Mgt):
    def __init__(self, n, l, m, r):
        datal = ConstVector (n-1, l)
        datam = ConstVector (n, m)
        datar = ConstVector (n-1, r)
        Mgt.__init__ (self, n, datal, datam, datar)
