#! /usr/bin/env python
#
# Copyright 2008  Jochen Voss <voss@seehuhn.de>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301 USA

"""Representation of symbolic expressions in Python.

This module implements the Value class.  A Value represents an
expression in a C-program and, when converted to a string, can be
directly used there.

Comparisons for Values are pessimistic, i.e. ``True`` is only returned
if the comparison is provably true.  Use of Values as booleans is
optimistic, i.e. ``False`` is returned only if the Value is known to
equal numeric zero.

The usual way to create Value objects is via the
:func:`~csimgen.value.value` function and via arithmetic operations.
If one of the operands in an arithmetic operation is a Value, the
other one is automatically cast to a Value, too.  Example:
``value("x")+3`` constructs a value representing the C expression
``x+3``.

Global functions
----------------

:func:`~csimgen.value.value`
    convert scalar python types into Values

Exported classes
----------------

:class:`~csimgen.value.Value`
    abstract base class
:class:`~csimgen.value.ValueAdd`
    represent the sum of values
:class:`~csimgen.value.ValueInv`
    represent the inverse of a value
:class:`~csimgen.value.ValueMult`
    represent the product of values
:class:`~csimgen.value.ValueNeg`
    represent the negative of a value
:class:`~csimgen.value.ValueNum`
    values which are known numbers
:class:`~csimgen.value.ValueVar`
    represent variables in the C program
"""

__author__ = 'Jochen Voss'
__version__ = '0.2'


def value(x, known_c=False):
    """Convert scalar python types to Value."""
    if isinstance(x, Value):
        return x
    elif isinstance(x, (float,int)):
        return ValueNum(x)
    elif isinstance(x, basestring):
        return ValueVar(x, known_c)
    else:
        raise TypeError("cannot convert %s to Value"%type(x))

def _snorm_simplify(a):
    """Simplify terms in semi-normal form."""
    terms = {}
    for p in a:
        key = frozenset((k,v) for k,v in p.iteritems() if k != "")
        if key in terms:
            terms[key][""] += p[""]
        else:
            terms[key] = p
    return [ p for p in terms.itervalues() if p[""] != 0 ]

def _snorm_add(a, b):
    """Add terms in semi-normal form."""
    return _snorm_simplify(a+b)

def _snorm_mult(a, b):
    """Multiply terms in semi-normal form."""
    res = []
    for s in a:
        for t in b:
            tmp = {}
            tmp[""] = s[""]*t[""]
            kk = set(s.keys()+t.keys())
            kk.remove("")
            for k in kk:
                tmp[k] = s.get(k, 0) + t.get(k, 0)
            res.append(tmp)
    return _snorm_simplify(res)

def _num_gcd(a, b):
    if isinstance(a, float):
        return a
    if isinstance(b, float):
        return b
    while b:
        a, b = b, a%b
    return a

def _snorm_gcd(a):
    gcd = {}
    for k,x in a[0].iteritems():
        gcd[k] = x
    for t in a[1:]:
        for k in set(t.keys()+gcd.keys()):
            if k == "":
                gcd[k] = _num_gcd(gcd.get(k,1), t.get(k,1))
            else:
                gcd[k] = min(gcd.get(k,0), t.get(k,0))
    return gcd

def _snorm_divide(a, factors):
    for t in a:
        remove = []
        for k in t:
            if k == "":
                t[k] /= factors[k]
            else:
                t[k] -= factors[k]
                if t[k] == 0:
                    remove.append(k)
        for k in remove:
            del t[k]
    return tuple(a)

def _norm_cancel(a, b):
    gcd = _snorm_gcd(a+b)
    if isinstance(gcd[""], float):
        gcd[""] = 1
    return _snorm_divide(a, gcd), _snorm_divide(b, gcd)

def _norm_sign(a):
    """Return the sign of a term in normal form.

    Possible results are '0', '>0', '>=0', '<0', '<=0', and '?'.
    """
    res = None
    for t in a:
        odd = sum([ t[k]%2 for k in t if k != "" ])
        if odd:
            return "?"
        x = t[""]
        if x == 0:
            sign = "0"
        elif x > 0 and len(t) == 1:
            sign = ">0"
        elif x < 0 and len(t) == 1:
            sign = "<0"
        elif x >= 0:
            sign = ">=0"
        elif x <= 0:
            sign = "<=0"

        if res is None or res == "0":
            res = sign
        elif res == sign or sign == "0":
            pass
        elif res in [ "<0", "<=0" ] and sign in [ ">0", ">=0" ]:
            return "?"
        elif res in [ ">0", ">=0" ] and sign in [ "<0", "<=0" ]:
            return "?"
        elif res == ">=0":
            res = ">0"
        elif res == "<=0":
            res = "<0"
    if res is None:
        res = "0"
    return res

def _snorm_as_value(a):
    terms = []
    for t in a:
        factors = [ValueNum(t[""])]
        for k in t:
            if k == "":
                continue
            factors.extend([ValueVar(k)]*t[k])
        terms.append(ValueMult(*factors))
    return ValueAdd(*terms)


class Value(object):

    """Abstract base class for all Values."""

    def __str__(self):
        return self._as_string()

    def protect(self, level, method="_as_string"):
        """Cast a Value into a string, adding enclosing brackets as needed.

        The argument `level` determines the required level of protection.
        Possible values are 1 (just cast to string), 2 (make safe for negation),
        3 (make safe for multiplication), and 4 (make safe for division).
        """
        s = getattr(self, method)()
        if self.stability < level:
            s = "(" + s + ")"
        return s

    def _is_zero(self):
        """Return True, if value is known to be 0."""
        x,y = self._norm()
        return _norm_sign(x) == "0"

    def _is_nonzero(self):
        """Return True, if value is known to be non-zero."""
        x,y = self._norm()
        return _norm_sign(x) in [ ">0", "<0" ]

    def _is_positive(self):
        x,y = self._norm()
        sx = _norm_sign(x)
        sy = _norm_sign(y)
        if sx == ">0" and sy in [ ">=0", ">0" ]:
            return True
        if sx == "<0" and sy in [ "<=0", "<0" ]:
            return True
        return False

    def _is_positive_or_zero(self):
        x,y = self._norm()
        sx = _norm_sign(x)
        sy = _norm_sign(y)
        if sx in [ "0", ">0", ">=0" ] and sy in [ ">=0", ">0" ]:
            return True
        if sx in [ "0", "<0", "<=0" ] and sy in [ "<=0", "<0" ]:
            return True
        return False

    def __nonzero__(self):
        """Return True, if value is not known to be 0."""
        return not self._is_zero()

    def __lt__(self, other):
        return (other-self)._is_positive()

    def __le__(self, other):
        return (other-self)._is_positive_or_zero()

    def __gt__(self, other):
        return (self-other)._is_positive()

    def __ge__(self, other):
        return (self-other)._is_positive_or_zero()

    def __eq__(self, other):
        return (self-other)._is_zero()

    def __ne__(self, other):
        return (self-other)._is_nonzero()

    def _as_number(self):
        x,y = self._norm()
        if len(x) == 0:
            xx = 0
        elif len(x) == 1:
            t = x[0]
            if len(t)>1:
                return None
            xx = t[""]
        else:
            return None
        if len(y) == 0:
            yy = 1
        elif len(y) == 1:
            t = y[0]
            if len(t)>1:
                return None
            yy = t[""]
        else:
            return None
        return xx/yy

    def __int__(self):
        x = self._as_number()
        if x is None:
            raise ValueError("cannot convert %s to int"%self)
        return int(x)

    def __long__(self):
        x = self._as_number()
        if x is None:
            raise ValueError("cannot convert %s to long"%self)
        return long(x)

    def __float__(self):
        x = self._as_number()
        if x is None:
            raise ValueError("cannot convert %s to float"%self)
        return float(x)

    def __add__(self, other):
        other = value(other)
        return ValueAdd(self, other)

    def __radd__(self, other):
        other = value(other)
        return ValueAdd(other, self)

    def __pos__(self):
        return self

    def __neg__(self):
        return ValueNeg(self)

    def __sub__(self, other):
        other = value(other)
        return ValueAdd(self, ValueNeg(other))

    def __rsub__(self, other):
        other = value(other)
        return ValueAdd(other, ValueNeg(self))

    def __mul__(self, other):
        try:
            other = value(other)
        except TypeError, e:
            try:
                return other.__rmul__(self)
            except (TypeError,AttributeError):
                raise e
        return ValueMult(self, other)

    def __rmul__(self, other):
        other = value(other)
        return ValueMult(other, self)

    def __div__(self, other):
        other = value(other)
        return ValueMult(self, ValueInv(other))

    __truediv__ = __div__

    def __rdiv__(self, other):
        other = value(other)
        return ValueMult(other, ValueInv(self))

    def __pow__(self, other):
        return ValueMult(*[self]*other)

    def _normalize(self):
        x,y = self._norm()
        res = ValueMult(_snorm_as_value(x), ValueInv(_snorm_as_value(y)))
        return res

    def _simplify_children(self):
        return self

    def simplify(self):
        """Try to simplify a term.

        This function tries several alternative representations of a
        term and returns the one with the shortest string
        representation.
        """

        if self.is_simple:
            return self
        if hasattr(self, "_simplify_cache"):
            return self._simplify_cache

        res = self
        res_len = len(str(res))

        x = self._simplify_children()

        tmp = x
        tmp_len = len(str(tmp))
        if tmp_len < res_len:
            res, res_len = tmp, tmp_len

        tmp = self._normalize()
        tmp_len = len(str(tmp))
        if tmp_len < res_len:
            res, res_len = tmp, tmp_len

        tmp = x._normalize()
        tmp_len = len(str(tmp))
        if tmp_len < res_len:
            res, res_len = tmp, tmp_len

        res.is_simple = True
        self._simplify_cache = res
        return res

    def __hash__(self):
        if hasattr(self, "_hash_cache"):
            return self._hash_cache

        xx, yy = self._norm()
        if len(yy) != 1 or yy[0] != { '': 1 }:
            raise NotImplementedError(repr(yy))

        # make sure we have hash(n) = hash(ValueNum(n)) for numbers
        res = 0
        for x in xx:
            hash(x[''])
            for k,v in x.iteritems():
                if k == '':
                    continue
                res ^= hash(k)^hash(v)

        self._hash_cache = res
        return res


class ValueNum(Value):

    """Values which are known numbers.

    This subclass of Value represents values which are numbers known
    to the python interpreter.
    """

    def __init__(self, val):
        """Create a value from a number.

        val -- a python number (either float or int)
        """
        self.stability = 5
        self.data = val
        self.is_simple = True

    def __repr__(self):
        return "ValueNum(%s)"%repr(self.data)

    def _as_string(self):
        return str(self.data)

    def _as_tex(self):
        return str(self.data)

    def _norm(self):
        return ([{"":self.data}], [{"":1}])

    def __pow__(self, other):
        return ValueNum(self.data**other)

    def is_const_C(self):
        return True


class ValueVar(Value):

    """Variables as values."""

    def __init__(self, name, known_c=False, tex_name=None):
        self.data = name
        self.known_c = known_c
        self.tex_name = tex_name

        self.stability = 6
        self.is_simple = True

    def __repr__(self):
        return "ValueVar(%s)"%repr(self.data)

    def _as_string(self):
        return self.data

    def _as_tex(self):
        if self.tex_name is not None:
            return self.tex_name
        s = self.data
        if len(s) > 1:
            return "\\"+s
        else:
            return s

    def _norm(self):
        name = self.data
        return ([{"":1, name:1}],[{"":1}])

    def is_const_C(self):
        return self.known_c


class ValueNeg(Value):

    """The negative of a Value."""

    def __new__(cls, arg):
        if isinstance(arg, ValueNeg):
            return arg.data
        elif isinstance(arg, ValueNum):
            return ValueNum(-arg.data)
        else:
            return super(ValueNeg, cls).__new__(cls)

    def __init__(self, arg):
        self.stability = 2
        self.data = arg
        self.is_simple = False

    def __repr__(self):
        return "ValueNeg(%s)"%repr(self.data)

    def _as_string(self):
        return "-"+self.data.protect(self.stability)

    def _as_tex(self):
        return "-"+self.data.protect(self.stability, method="_as_tex")

    def _simplify_children(self):
        return ValueNeg(self.data.simplify())

    def _norm(self):
        numerator,denominator = self.data._norm()
        tmp = []
        for term in numerator:
            term = term.copy()
            term[""] = -term[""]
            tmp.append(term)
        return tmp,denominator

    def is_const_C(self):
        return self.data.is_const_C()


class ValueAdd(Value):

    """The sum of Values."""

    def __new__(cls, *args):
        # clean up the list of arguments
        tmp = []
        for x in args:
            if isinstance(x, ValueAdd):
                tmp.extend(x.data)
            else:
                tmp.append(x)
        args = []
        num = 0
        for x in tmp:
            if isinstance(x, ValueNum):
                num += x.data
            else:
                args.append(x)

        # handle special cases
        if not args:
            return ValueNum(num)
        if num:
            args.append(ValueNum(num))
        if len(args) == 1:
            return args[0]

        res = super(ValueAdd, cls).__new__(cls)
        res.data = tuple(args)
        return res

    def __init__(self, *args):
        self.stability = 1
        # self.data is set by __new__
        self.is_simple = False

    def __repr__(self):
        argstr = ",".join([repr(x) for x in self.data])
        return "ValueAdd(%s)"%argstr

    def _as(self, method):
        data = self.data
        if not data:
            data = [ValueNum(0)]
        res = [data[0].protect(self.stability, method)]
        for d in data[1:]:
            s = d.protect(self.stability, method)
            if s[0] != '-':
                res.append ("+")
            res.append (s)
        return "".join(res)

    def _as_string(self):
        return self._as("_as_string")

    def _as_tex(self):
        return self._as("_as_tex")

    def _simplify_children(self):
        children = [ x.simplify() for x in self.data ]
        return ValueAdd(*children)

    def _norm(self):
        if hasattr(self, "_norm_cache"):
            return self._norm_cache

        a = [{"":0}]
        b = [{"":1}]
        for s in self.data:
            # add c/d
            c,d = s._norm()
            ad = _snorm_mult(a, d)
            bc = _snorm_mult(b, c)
            bd = _snorm_mult(b, d)
            a,b = _snorm_add(ad, bc), bd

        self._norm_cache = _norm_cancel(a, b)
        return self._norm_cache

    def is_const_C(self):
        for x in self.data:
            if not x.is_const_C():
                return False
        return True


class ValueInv(Value):

    """The inverse of a Value."""

    def __new__(cls, arg):
        if isinstance(arg, ValueInv):
            return arg.data
        elif isinstance(arg, ValueNum):
            if arg.data == 1:
                return arg
            elif isinstance(arg.data, float):
                return ValueNum(1.0/arg.data)
        elif isinstance(arg, ValueNeg):
            return ValueNeg(ValueInv(arg.data))

        res = super(ValueInv, cls).__new__(cls)
        res.data = arg
        return res

    def __init__(self, arg):
        self.stability = 4
        # self.data is set by __new__
        self.is_simple = False

    def __repr__(self):
        return "ValueInv(%s)"%repr(self.data)

    def _as_string(self):
        return "1.0/"+self.data.protect(self.stability)

    def _as_tex(self):
        return "\\frac{1}{"+self.data._as_tex()+"}"

    def _simplify_children(self):
        return ValueInv(self.data.simplify())

    def _norm(self):
        numerator,denominator = self.data._norm()
        return _norm_cancel(denominator, numerator)

    def is_const_C(self):
        return self.data.is_const_C()


class ValueMult(Value):

    """The product of Values."""

    def __new__(cls, *args):
        def _p(x, data, div, positive):
            if isinstance(x, ValueNeg):
                positive = not positive
                x = x.data
            if isinstance(x, ValueMult):
                for y in x.data:
                    positive = _p(y, data, div, positive)
            elif isinstance(x, ValueInv):
                positive = _p(x.data, div, data, positive)
            else:
                data.append(x)
            return positive
        data = []
        div = []
        positive = True
        for x in args:
            positive = _p(x, data, div, positive)

        # cancel common terms
        if len(div)<len(data):
            cancel = [ x for x in div if x in data ]
        else:
            cancel = [ x for x in data if x in div ]
        for x in cancel:
            while x in data and x in div:
                data.remove(x)
                div.remove(x)

        # extract all numeric values
        num_numerator = []
        num_denominator = []
        tmp = []
        for x in data:
            if isinstance(x, ValueNum):
                num_numerator.append(x.data)
            else:
                tmp.append(x)
        data = tmp
        tmp = []
        for x in div:
            if isinstance(x, ValueNum):
                num_denominator.append(x.data)
            else:
                tmp.append(x)
        div = tmp

        # cancel numeric factors
        numerator = reduce(lambda x,y: x*y, num_numerator, 1)
        if numerator == 0:
            return ValueNum(0)
        denominator = reduce(lambda x,y: x*y, num_denominator, 1)
        if numerator < 0:
            positive = not positive
            numerator = -numerator
        if denominator < 0:
            positive = not positive
            denominator = - denominator
        if isinstance(numerator, float) or isinstance(denominator, float):
            numerator = numerator/denominator
            denominator = 1
        else:
            b = _num_gcd(numerator, denominator)
            numerator /= b
            denominator /= b

        # merge everything
        if denominator != 1:
            div[0:0] = [ValueNum(denominator)]
        if len(div)>1:
            data.append(ValueInv(ValueMult(*div)))
        elif len(div)==1:
            data.append(ValueInv(div[0]))
        if not positive and (numerator != 1 or not data):
            numerator = -numerator
            positive = True
        if numerator != 1 or not data:
            data[0:0] = [ValueNum(numerator)]

        # construct the result
        if len(data) == 1:
            if positive:
                return data[0]
            else:
                return ValueNeg(data[0])
        if not positive:
            return ValueNeg(ValueMult(*data))
        res = super(ValueMult, cls).__new__(cls)
        res.data = tuple(data)
        return res

    def __init__(self, *args):
        self.stability = 3
        # self.data is set by __new__
        self.is_simple = False

    def __repr__(self):
        argstr = ",".join([repr(x) for x in self.data])
        return "ValueMult(%s)"%argstr

    def _as_string(self):
        data = self.data
        if not data:
            data = [ValueNum(1)]
        res = [data[0].protect(self.stability)]
        for d in data[1:]:
            s = d.protect(self.stability)
            if s.startswith ("1.0/"):
                res.append (s[3:])
            else:
                res.append ("*"+s)
        return "".join(res)

    def _as_tex(self):
        numerator = []
        denominator = []
        for x in self.data:
            if isinstance(x, ValueInv):
                denominator.append(x.data)
            else:
                numerator.append(x)

        if len(numerator)>1:
            numerator = [ x.protect(self.stability,"_as_tex") for x in numerator ]
        else:
            numerator = [ x._as_tex() for x in numerator ]
        if len(denominator)>1:
            denominator = [ x.protect(self.stability,"_as_tex") for x in denominator ]
        else:
            denominator = [ x._as_tex() for x in denominator ]

        def _n(xx):
            res = []
            count = 1
            for i in range(0,len(xx)):
                if i<len(xx)-1 and xx[i] == xx[i+1]:
                    count += 1
                else:
                    if count>=10:
                        res.append(xx[i]+"^{%d}"%count)
                    elif count>1:
                        res.append(xx[i]+"^%d"%count)
                    else:
                        res.append(xx[i])
                    count = 1
            return res
        n_str = " ".join(_n(numerator))
        if n_str[0] == "-":
            n_str = n_str[1:]
            sign = "-"
        else:
            sign = ""
        d_str = " ".join(_n(denominator))

        if d_str:
            return "%s\\frac{%s}{%s}"%(sign,n_str,d_str)
        else:
            return n_str

    def _simplify_children(self):
        children = [ x.simplify() for x in self.data ]
        return ValueMult(*children)

    def _norm(self):
        if hasattr(self, "_norm_cache"):
            return self._norm_cache

        a = [{"":1}]
        b = [{"":1}]
        for s in self.data:
            # multiply with c/d
            c,d = s._norm()
            ac = _snorm_mult(a, c)
            bd = _snorm_mult(b, d)
            a,b = ac,bd

        self._norm_cache = _norm_cancel(a, b)
        return self._norm_cache

    def is_const_C(self):
        for x in self.data:
            if not x.is_const_C():
                return False
        return True
