#! /usr/bin/env python
#
# Copyright 2007  Jochen Voss <voss@seehuhn.de>
#
# This file is part of the CSimGen package.
#
# CSimGen is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301 USA

"""Vector representation inside the code generator.

This module implements a Vector class.  The vectors are vectors in the
python code generator, the elements of the vectors are Values.  The
sub-class 'CVector' is used to represent vectors from the
generated C source inside the code generator.

Note that functions of this module never generate C code.  Assignments
to vector elements change the vector represented in the Python code
generator, they do not change vector elements in the generated code
(use instances of CodeGen to do the latter).

WARNING: this is an experimental version of CSimGen.  The API is not
yet finalised and may change without further notice.
"""

__author__ = 'Jochen Voss'
__version__ = '0.1'

from value import value
from copy import copy

def cumsum(vec):
    """Calculate the cumulative sum of a sequence or vector."""
    res = []
    x = 0
    for val in vec:
        x += val
        res.append (x)
    return  res

def _slicify(idx,m0,m1):
    """Unify access via slice objects or indices.

    The input value 'idx' must be either a slice object or a Value.
    The values 'm0' and 'm1' give the indexed range.  The function
    returns a triple (a,b,c) of Value objects where 'a' is the lower
    bound of the range, 'b' is the upper bound, and 'c' is the step
    increment.
    """
    if isinstance(idx,slice):
        i0,i1,istep = (idx.start,idx.stop,idx.step)
        if i0 is None: i0 = m0
        if i1 is None: i1 = m1
        if istep is None: istep = 1
    else:
        i0 = value(idx)
        i1 = i0+1
        istep = 1
    return map(value,[i0,i1,istep])

class Vector (object):

    """Vector representation inside the Python code generator."""

    def __init__(self, n, offset=0):
        self.n = value(n).simplify()
        self.offset = offset
        self.items = {}

    def __getitem__(self, i):
        if isinstance (i, slice):
            i0, i1, istep = _slicify (i,self.offset,self.offset+self.n)
            return  SubVector (self, i0, i1, istep)
        i = value(i)
        if i<self.offset or i>=self.offset+self.n:
            tmpl = "%s outside the allowed range %s..%s"
            raise IndexError(tmpl%(i,self.offset,self.offset+self.n))
        if i in self.items:
            return  self.items[i]
        else:
            return  self.get (i)

    def __setitem__(self, i, val):
        i = value(i)
        if i<self.offset or i>=self.offset+self.n:
            raise IndexError()
        val = value (val)
        if hasattr (self, "set"):
            self.set (i, val)
        else:
            self.items[i] = val

    def __pos__(self):
        return  self

    def __neg__(self):
        return  FnVector (self, lambda x: -x)

    def __add__(self, other):
        return  BinopVector (self, other, lambda x, y: x+y)

    def __sub__(self, other):
        return  BinopVector (self, other, lambda x, y: x-y)

    def __mul__(self, other):
        try:
            x = value(other)
        except TypeError:
            raise TypeError("cannot multiply Vector and %s"%type(other))
        return  FnVector (self, lambda val: val*x)

    def __div__(self, other):
        x = value(other)
        return  FnVector (self, lambda val: val/x)

    def __rmul__(self, other):
        x = value(other)
        return  FnVector (self, lambda val: x*val)

    def pmult(self, other):
        return  BinopVector (self, other, lambda x, y: x*y)

    def special(self):
        """Return the list of indices which are special cased."""
        return  set(self.items.keys())

class ConcatVector (Vector):
    def __init__(self, *pieces):
        lengths = [p.n for p in pieces]
        n = sum(lengths)
        Vector.__init__ (self, n)
        self.breaks = cumsum (lengths)
        self.pieces = pieces

    def _find_piece(self, i):
        i = value(i)
        old = 0
        for k,next in enumerate(self.breaks):
            if i>=old and i<next:
                return i-old,self.pieces[k]
            old = next
        raise IndexError()

    def get(self, i):
        ii,xx = self._find_piece (i)
        return xx[ii]

    def set(self, i, val):
        ii,xx = self._find_piece (i)
        xx[ii] = val

class SubVector (Vector):
    def __init__(self, x, i0, i1, istep):
        #if i0 < 0 or i1 > x.n: raise IndexError ()
        Vector.__init__ (self, (i1-i0)/istep)
        self.x = x
        self.i0, self.i1, self.istep = map(value, [i0, i1, istep])

    def get(self, i):
        return  self.x[self.i0+i*self.istep]

    def set(self, i, val):
        self.x[self.i0+i*self.istep] = val

    def base_and_inc(self):
        xb,xi = self.x.base_and_inc ()
        return  (xb+self.i0*xi,self.istep*xi)

class BinopVector (Vector):
    def __init__(self, x, y, fn):
        n = x.n
        if y.n != n or y.offset != x.offset:
            raise ValueError("vectors must have equal index ranges")
        Vector.__init__ (self, n)
        self.x = x
        self.y = y
        self.fn = fn

    def get(self, i):
        return  self.fn (self.x[i],self.y[i])

    def special(self):
        return  Vector.special(self) | self.x.special() | self.y.special()

class FnVector (Vector):

    """Modify a vector by applying a function to each element.

    This class represents vectors obtained by transforming each
    element of another vector.
    """

    def __init__(self, x, fn):
        """Construct a new instance of the FnVector class.

        'x' must be a Vector, and 'fn' must be a function which maps
        Value to Value.
        """
        Vector.__init__ (self, x.n)
        self.x = x
        self.fn = fn

    def get(self, i):
        return  self.fn (self.x[i])

    def special(self):
        return  Vector.special(self) | self.x.special()

class ExplicitVector (Vector):
    def __init__(self, n, values):
        Vector.__init__ (self, n)
        for i,val in enumerate(values):
            self.items[value(i)] = value(val)

    def get(self, i):
        i = value(i)
        raise NotImplementedError()

class CVector (Vector):
    def __init__(self, n, name, base=0, inc=1):
        Vector.__init__ (self, n)
        self.name = name
        self.base = base
        self.inc = inc

    def __getitem__(self, i):
        if isinstance (i, slice):
            i0, i1, istep = _slicify(i, self.offset, self.offset+self.n)
            return CVector((i1-i0)/istep, self.name,
                           self.base+self.inc*i0, self.inc*istep)
        return Vector.__getitem__(self, i)

    def get(self, i):
        idx = (i*self.inc + self.base).simplify()
        return value("%s[%s]"%(self.name,idx))

    def spec(self):
        assert  self.base == 0
        assert  self.inc == 1
        return (self.name, "double", self.n)

    def base_and_inc(self):
        return  (value(self.name)+self.base, self.inc)

    def __str__(self):
        assert  self.base == 0
        assert  self.inc == 1
        return  str(self.name)

class ComputedVector (Vector):
    def __init__(self, n, fn):
        Vector.__init__ (self, n)
        self.fn = fn

    def get(self, i):
        return  self.fn(i)

    def special(self):
        return  Vector.special(self)

class ConstVector (ComputedVector):
    def __init__(self, n, val):
        val = value(val)
        ComputedVector.__init__ (self, n, lambda i: val)

class AffineVector (ComputedVector):

    """Vectors where the components are an affine function of the index.

    This is the Vector equivalent of the python function range.
    """

    def __init__(self, n, a=1, b=0):
        """Construct a new vector of size n with x[i] = a*i+b."""
        a = value(a)
        b = value(b)
        ComputedVector.__init__ (self, n, lambda i: a*i+b)
