/* dgbtrs.h - C language wrapper for the LAPACK routine DGBTRS
 *
 * Copyright (C) 2006  Jochen Voss.
 *
 * Purpose
 * =======
 *
 * DGBTRS solves a system of linear equations
 *    A * X = B  or  A' * X = B
 * with a general band matrix A using the LU factorization computed
 * by DGBTRF.
 *
 * Arguments
 * =========
 *
 * TRANS   (input) CHARACTER*1
 *         Specifies the form of the system of equations.
 *         = 'N':  A * X = B  (No transpose)
 *         = 'T':  A'* X = B  (Transpose)
 *         = 'C':  A'* X = B  (Conjugate transpose = Transpose)
 *
 * N       (input) INTEGER
 *         The order of the matrix A.  N >= 0.
 *
 * KL      (input) INTEGER
 *         The number of subdiagonals within the band of A.  KL >= 0.
 *
 * KU      (input) INTEGER
 *         The number of superdiagonals within the band of A.  KU >= 0.
 *
 * NRHS    (input) INTEGER
 *         The number of right hand sides, i.e., the number of columns
 *         of the matrix B.  NRHS >= 0.
 *
 * AB      (input) DOUBLE PRECISION array, dimension (LDAB,N)
 *         Details of the LU factorization of the band matrix A, as
 *         computed by DGBTRF.  U is stored as an upper triangular band
 *         matrix with KL+KU superdiagonals in rows 1 to KL+KU+1, and
 *         the multipliers used during the factorization are stored in
 *         rows KL+KU+2 to 2*KL+KU+1.
 *
 * LDAB    (input) INTEGER
 *         The leading dimension of the array AB.  LDAB >= 2*KL+KU+1.
 *
 * IPIV    (input) INTEGER array, dimension (N)
 *         The pivot indices; for 1 <= i <= N, row i of the matrix was
 *         interchanged with row IPIV(i).
 *
 * B       (input/output) DOUBLE PRECISION array, dimension (LDB,NRHS)
 *         On entry, the right hand side matrix B.
 *         On exit, the solution matrix X.
 *
 * LDB     (input) INTEGER
 *         The leading dimension of the array B.  LDB >= max(1,N).
 *
 * INFO    (output) INTEGER
 *         = 0:  successful exit
 *         < 0: if INFO = -i, the i-th argument had an illegal value
 */

#ifndef FILE_DGBTRS_H_SEEN
#define FILE_DGBTRS_H_SEEN

static inline long
dgbtrs (char TRANS, long N, long KL, long KU, long NRHS, const double *AB,
        long LDAB, const long *IPIV, double *B, long LDB)
{
  extern  void  dgbtrs_ (const char *TRANSp, const long *Np, const long *KLp,
                         const long *KUp, const long *NRHSp, const double *AB,
                         const long *LDABp, const long *IPIV, double *B,
                         const long *LDBp, long *INFOp);
  long  info;
  dgbtrs_ (&TRANS, &N, &KL, &KU, &NRHS, AB, &LDAB, IPIV, B, &LDB, &info);
  return info;
}

#endif /* FILE_DGBTRS_H_SEEN */
