/* dgetrf.h - C language wrapper for the LAPACK routine DGETRF
 *
 * Copyright (C) 2006  Jochen Voss.
 *
 * Purpose
 * =======
 *
 * DGETRF computes an LU factorization of a general M-by-N matrix A
 * using partial pivoting with row interchanges.
 *
 * The factorization has the form
 *    A = P * L * U
 * where P is a permutation matrix, L is lower triangular with unit
 * diagonal elements (lower trapezoidal if m > n), and U is upper
 * triangular (upper trapezoidal if m < n).
 *
 * This is the right-looking Level 3 BLAS version of the algorithm.
 *
 * Arguments
 * =========
 *
 * M       (input) INTEGER
 *         The number of rows of the matrix A.  M >= 0.
 *
 * N       (input) INTEGER
 *         The number of columns of the matrix A.  N >= 0.
 *
 * A       (input/output) DOUBLE PRECISION array, dimension (LDA,N)
 *         On entry, the M-by-N matrix to be factored.
 *         On exit, the factors L and U from the factorization
 *         A = P*L*U; the unit diagonal elements of L are not stored.
 *
 * LDA     (input) INTEGER
 *         The leading dimension of the array A.  LDA >= max(1,M).
 *
 * IPIV    (output) INTEGER array, dimension (min(M,N))
 *         The pivot indices; for 1 <= i <= min(M,N), row i of the
 *         matrix was interchanged with row IPIV(i).
 *
 * INFO    (output) INTEGER
 *         = 0:  successful exit
 *         < 0:  if INFO = -i, the i-th argument had an illegal value
 *         > 0:  if INFO = i, U(i,i) is exactly zero. The factorization
 *               has been completed, but the factor U is exactly
 *               singular, and division by zero will occur if it is used
 *               to solve a system of equations.
 */

#ifndef FILE_DGETRF_H_SEEN
#define FILE_DGETRF_H_SEEN

static inline long
dgetrf (long M, long N, double *A, long LDA, long *IPIV)
{
  extern  void  dgetrf_ (const long *Mp, const long *Np, double *A,
                         const long *LDAp, long *IPIV, long *INFOp);
  long  info;
  dgetrf_ (&M, &N, A, &LDA, IPIV, &info);
  return info;
}

#endif /* FILE_DGETRF_H_SEEN */
