#! /usr/bin/env python

from csimgen.value import *
import unittest


class TestSimplify(unittest.TestCase):

    def setUp(self):
        self.one = ValueNum(1)
        self.two = ValueNum(2)
        self.three = ValueNum(3)
        self.pi = ValueNum(3.0)
        self.x = ValueVar('x')
        self.y = ValueVar('y')
        self.z = ValueVar('z')

    def test_a(self):
        """simplifications in the __new__ methods"""
        atests = [
            (self.one, 'ValueNum(1)'),
            (self.pi, 'ValueNum(3.0)'),

            (self.x, "ValueVar('x')"),

            (-self.x, "ValueNeg(ValueVar('x'))"),
            (--self.x, "ValueVar('x')"),
            (-self.one, "ValueNum(-1)"),
            (-self.pi, "ValueNum(-3.0)"),

            (ValueAdd(), "ValueNum(0)"),
            (ValueAdd(self.x), "ValueVar('x')"),
            (self.x+self.y, "ValueAdd(ValueVar('x'),ValueVar('y'))"),
            (ValueAdd(self.x,self.y,self.z),
             "ValueAdd(ValueVar('x'),ValueVar('y'),ValueVar('z'))"),
            (self.x+(self.y+self.z),
             "ValueAdd(ValueVar('x'),ValueVar('y'),ValueVar('z'))"),
            ((self.x+self.y)+self.z,
             "ValueAdd(ValueVar('x'),ValueVar('y'),ValueVar('z'))"),
            (ValueAdd(), "ValueNum(0)"),
            (ValueAdd(self.one), "ValueNum(1)"),
            (self.one+self.one, "ValueNum(2)"),
            (self.one+self.pi, "ValueNum(4.0)"),
            (ValueAdd(self.one,self.x,self.one),
             "ValueAdd(ValueVar('x'),ValueNum(2))"),

            (ValueInv(self.x), "ValueInv(ValueVar('x'))"),
            (ValueInv(ValueInv(self.x)), "ValueVar('x')"),
            (ValueInv(self.one), "ValueNum(1)"),
            (ValueInv(self.two), "ValueInv(ValueNum(2))"),
            (ValueInv(self.pi), "ValueNum(%s)"%(repr(1/3.0))),
            (ValueInv(-self.x), "ValueNeg(ValueInv(ValueVar('x')))"),

            (ValueMult(), "ValueNum(1)"),
            (ValueMult(self.x), "ValueVar('x')"),
            (ValueMult(-self.x), "ValueNeg(ValueVar('x'))"),
            (self.x*self.y, "ValueMult(ValueVar('x'),ValueVar('y'))"),
            (ValueMult(self.x,self.y,self.z),
             "ValueMult(ValueVar('x'),ValueVar('y'),ValueVar('z'))"),
            (self.x*(-self.y),
             "ValueNeg(ValueMult(ValueVar('x'),ValueVar('y')))"),
            (self.x*(self.y*self.z),
             "ValueMult(ValueVar('x'),ValueVar('y'),ValueVar('z'))"),
            ((self.x*self.y)*self.z,
             "ValueMult(ValueVar('x'),ValueVar('y'),ValueVar('z'))"),
            (self.two*ValueInv(self.x*(self.y*self.z)),
             "ValueMult(ValueNum(2),ValueInv(ValueMult(ValueVar('x'),ValueVar('y'),ValueVar('z'))))"),
            (self.x*ValueInv(self.x*self.y), "ValueInv(ValueVar('y'))"),
            (self.x*self.y*ValueInv(self.x), "ValueVar('y')"),
            (self.two*self.three, "ValueNum(6)"),
            (ValueNum(-2)/ValueNum(3),
             "ValueMult(ValueNum(-2),ValueInv(ValueNum(3)))"),
            (ValueNum(-2)/ValueNum(-3),
             "ValueMult(ValueNum(2),ValueInv(ValueNum(3)))"),
            (ValueNum(2)/ValueNum(-3),
             "ValueMult(ValueNum(-2),ValueInv(ValueNum(3)))"),
            (self.two*self.three/ValueNum(10),
             "ValueMult(ValueNum(3),ValueInv(ValueNum(5)))"),
            (ValueNum(10)/(self.two*self.three),
             "ValueMult(ValueNum(5),ValueInv(ValueNum(3)))"),
            (self.two*ValueNum(3.0)/ValueNum(10), "ValueNum(%s)"%(repr(6.0/10))),
            (ValueMult(self.one,self.two,self.x,self.y,ValueInv(self.two)),
             "ValueMult(ValueVar('x'),ValueVar('y'))"),
            (ValueMult(self.one,self.two,self.x,self.y),
             "ValueMult(ValueNum(2),ValueVar('x'),ValueVar('y'))"),
            (ValueMult(self.one,self.two,self.x,ValueInv(self.y)),
             "ValueMult(ValueNum(2),ValueVar('x'),ValueInv(ValueVar('y')))"),
            (ValueMult(self.one,self.two,self.x,ValueInv(self.y*self.z)),
             "ValueMult(ValueNum(2),ValueVar('x'),ValueInv(ValueMult(ValueVar('y'),ValueVar('z'))))"),
            (ValueMult(self.one,-self.x,self.y),
             "ValueNeg(ValueMult(ValueVar('x'),ValueVar('y')))"),
            (ValueMult(self.one,-self.x,1/self.x),
             "ValueNum(-1)"),
            (ValueMult(self.two,-self.x,self.y),
             "ValueMult(ValueNum(-2),ValueVar('x'),ValueVar('y'))"),
            (ValueMult(self.x), "ValueVar('x')"),
            (ValueMult(-self.x), "ValueNeg(ValueVar('x'))"),
        ]
        for k,test in enumerate(atests):
            a,b = test
            self.failUnless(repr(a) == b)


class TestComparisons(unittest.TestCase):

    def setUp(self):
	self.x = value("x")
	self.y = value("y")

    def test_eq1(self):
	"Symbols equal themselves."
	self.failUnless(self.x == self.x)

    def test_eq2(self):
	"Differing symbols are not equal."
	self.failIf(self.x == self.y)

    def test_ne1(self):
	"Symbols are not different from themselves."
	self.failIf(self.x != self.x)

    def test_ne2(self):
	"Differing symbols are incomparable."
	self.failIf(self.x != self.y)

    def test_geq1(self):
	"Symbols are greater or equal to themselves."
	self.failUnless(self.x >= self.x)

    def test_geq2(self):
	"If symbol x differes from symbol y, then x >= y does not hold."
	self.failIf(self.x >= self.y)

    def test_leq1(self):
	"Symbols are less or equal to themselves."
	self.failUnless(self.x <= self.x)

    def test_leq2(self):
	"If symbol x differes from symbol y, then x <= y does not hold."
	self.failIf(self.x <= self.y)

    def test_gt1(self):
	"Symbols are not greater than themselves."
	self.failIf(self.x > self.x)

    def test_gt2(self):
	"If symbol x differes from symbol y, then x > y does not hold."
	self.failIf(self.x > self.y)

    def test_lt1(self):
	"Symbols are not less than themselves."
	self.failIf(self.x < self.x)

    def test_lt2(self):
	"If symbol x differes from symbol y, then x < y does not hold."
	self.failIf(self.x < self.y)


class TestPathological(unittest.TestCase):

    def setUp(self):
	self.a = ValueAdd()
	self.b = ValueMult()

    def test_empty_sum1(self):
	"Empty sum is 0."
	self.failUnless(self.a == 0)

    def test_empty_sum2(self):
	"Convert emtpy sum to string."
	self.failUnless (str(self.a) == "0")

    def test_empty_product1(self):
	"Empty product is 1."
	self.failUnless(self.b == 1)

    def test_empty_product2(self):
	"Convert emtpy product to string."
	self.failUnless (str(self.b) == "1")


class TestNormalize(unittest.TestCase):

    @staticmethod
    def _teststr(x):
        results = [ x<0, x<=0, x==0, x>=0, x>0 ]
        return "".join(map(lambda s: s and "1" or "0", results))

    def setUp(self):
	self.x = value("x")
	self.y = value("y")

    def test_zero(self):
        "zero equals zero"
        self.failUnless(self._teststr(value(0)) == "01110")

    def test_one(self):
        "one is strictly positive"
        self.failUnless(self._teststr(value(1)) == "00011")

    def test_minusone(self):
        "minus one is strictly negative"
        self.failUnless(self._teststr(value(-1)) == "11000")

    def test_x(self):
        "x does not compare with 0"
        x = self.x
        self.failUnless(self._teststr(x) == "00000")

    def test_xx(self):
        "x^2 is positive"
        x = self.x
        self.failUnless(self._teststr(x*x) == "00010")

    def test_xx_plus_yy(self):
        "x^2+y^2 is positive"
        x = self.x
        y = self.y
        self.failUnless(self._teststr(x*x+y*y) == "00010")

    def test_xx_minus_yy(self):
        "x^2-y^2 does not compare with 0"
        x = self.x
        y = self.y
        self.failUnless(self._teststr(x*x-y*y) == "00000")

    def test_xx_plus_one(self):
        "x^2+1 is strictly positive"
        x = self.x
        self.failUnless(self._teststr(x*x+1) == "00011")

    def test_xx_minus_one(self):
        "x^2-1 does not compare with 0"
        x = self.x
        self.failUnless(self._teststr(x*x-1) == "00000")

    def test_minus_x(self):
        "-x does not compare with 0"
        x = self.x
        self.failUnless(self._teststr(-x) == "00000")

    def test_minus_xx(self):
        "x^2 is negative"
        x = self.x
        self.failUnless(self._teststr(-x*x) == "01000")

    def test_minus_xx_plus_one(self):
        "-x^2-1 is strictly negative"
        x = self.x
        self.failUnless(self._teststr(-x*x-1) == "11000")

    def test_minus_xx_minus_one(self):
        "-x^2+1 does not compare with 0"
        x = self.x
        self.failUnless(self._teststr(-x*x+1) == "00000")

    def test_div1(self):
        "1/x^2 is strictly positive"
        x = self.x
        self.failUnless(self._teststr(1/(x*x)) == "00011")

    def test_div2(self):
        "x^2/y^2 is positive"
        x = self.x
        y = self.y
        self.failUnless(self._teststr((x*x)/(y*y)) == "00010")

    def test_div3(self):
        "(x^2+1)/x^2-1 is strictly positive"
        x = self.x
        self.failUnless(self._teststr((x*x+1)/(x*x)-1) == "00011")


class TestBoolean(unittest.TestCase):
    def setUp(self):
	self.x = value("x")

    def test_zero(self):
        self.failUnless (not value(0))

    def test_one(self):
        self.failUnless (not not value(1))

    def test_x(self):
        self.failUnless (not not self.x)


class TestNumeric(unittest.TestCase):

    def setUp(self):
	self.x = value("x")

    def test_zero(self):
        "int(value(0)) == 0"
        self.failUnless (int(value(0)) == 0)

    def test_one(self):
        "int(value(1)) == 1"
        self.failUnless (int(value(1)) == 1)

    def test_sub(self):
        "int(x-x) == 0"
        x = self.x
        self.failUnless (int(x-x) == 0)

    def test_div(self):
        "int(x/x) == 1"
        x = self.x
        self.failUnless (int(x/x) == 1)

if __name__ == "__main__":
    unittest.main ()
